@@ -175,9 +175,6 @@ int AB_Banking_ExecuteJobs(AB_BANKING *ab, AB_JOB_LIST2 *jl2,
     AB_Job_List2Iterator_free(jit);
   }
 
-  /* clear temporarily accepted certificates */
-  GWEN_DB_ClearGroup(ab->dbTempConfig, "certificates");
-
   /* execute jobs */
   pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_ALLOW_SUBLEVELS |
 			     GWEN_GUI_PROGRESS_SHOW_PROGRESS |
@@ -198,9 +195,6 @@ int AB_Banking_ExecuteJobs(AB_BANKING *ab, AB_JOB_LIST2 *jl2,
     DBG_INFO(AQBANKING_LOGDOMAIN, "here (%d)", rv);
   }
 
-  /* clear temporarily accepted certificates again */
-  GWEN_DB_ClearGroup(ab->dbTempConfig, "certificates");
-
   /* clear queue */
   GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice,
 		       I18N("Postprocessing jobs"));
@@ -260,93 +254,6 @@ int AB_Banking_ExecuteJobs(AB_BANKING *ab, AB_JOB_LIST2 *jl2,
 
 
 
-int AB_Banking_ActivateProvider(AB_BANKING *ab, const char *pname) {
-  AB_PROVIDER *pro;
-
-  if (GWEN_StringList_HasString(ab->activeProviders, pname)) {
-    DBG_INFO(AQBANKING_LOGDOMAIN, "Provider already active");
-    return GWEN_ERROR_FOUND;
-  }
-
-  pro=AB_Banking_GetProvider(ab, pname);
-  if (!pro) {
-    DBG_ERROR(AQBANKING_LOGDOMAIN,
-              "Could not load backend \"%s\"", pname);
-    return GWEN_ERROR_NOT_FOUND;
-  }
-
-  GWEN_StringList_AppendString(ab->activeProviders, pname, 0, 1);
-  return 0;
-}
-
-
-
-int AB_Banking_DeactivateProvider(AB_BANKING *ab, const char *pname) {
-  AB_ACCOUNT *a;
-  AB_USER *u;
-  AB_PROVIDER *pro;
-
-  if (!GWEN_StringList_HasString(ab->activeProviders, pname)) {
-    DBG_INFO(AQBANKING_LOGDOMAIN, "Provider not active");
-    return GWEN_ERROR_INVALID;
-  }
-
-  pro=AB_Banking_FindProvider(ab, pname);
-  if (pro)
-    AB_Banking_FiniProvider(ab, pro);
-
-  GWEN_StringList_RemoveString(ab->activeProviders, pname);
-
-  /* delete accounts which use this backend */
-  a=AB_Account_List_First(ab->accounts);
-  while(a) {
-    AB_ACCOUNT *na;
-
-    na=AB_Account_List_Next(a);
-    pro=AB_Account_GetProvider(a);
-    assert(pro);
-    if (strcasecmp(AB_Provider_GetName(pro), pname)==0) {
-      AB_Account_List_Del(a);
-      AB_Account_free(a);
-    }
-    a=na;
-  }
-
-  /* delete users which use this backend */
-  u=AB_User_List_First(ab->users);
-  while(u) {
-    AB_USER *nu;
-    const char *s;
-
-    nu=AB_User_List_Next(u);
-    s=AB_User_GetBackendName(u);
-    assert(s && *s);
-    if (strcasecmp(s, pname)==0) {
-      AB_User_List_Del(u);
-      AB_User_free(u);
-    }
-    u=nu;
-  }
-
-  return 0;
-}
-
-
-
-int AB_Banking_IsProviderActive(AB_BANKING *ab, const char *backend){
-  AB_PROVIDER *pro;
-
-  pro=AB_Banking_FindProvider(ab, backend);
-  if (!pro) {
-    DBG_ERROR(AQBANKING_LOGDOMAIN, "Provider \"%s\" not found", backend);
-    return 0;
-  }
-
-  return AB_Provider_IsInit(pro);
-}
-
-
-
 const GWEN_STRINGLIST *AB_Banking_GetActiveProviders(const AB_BANKING *ab) {
   assert(ab);
   if (GWEN_StringList_Count(ab->activeProviders)==0)
@@ -699,91 +606,16 @@ GWEN_PLUGIN_DESCRIPTION_LIST2 *AB_Banking_GetDebuggerDescrs(AB_BANKING *ab,
 
 
 
-int AB_Banking__LoadOldProviderData(AB_BANKING *ab, const char *name){
-  GWEN_BUFFER *pbuf;
-  GWEN_DB_NODE *db;
-
-  assert(ab);
-  pbuf=GWEN_Buffer_new(0, 256, 0, 1);
-
-  if (AB_Banking_GetUserDataDir(ab, pbuf)) {
-    DBG_ERROR(AQBANKING_LOGDOMAIN, "Could not get user data dir");
-    GWEN_Buffer_free(pbuf);
-    return GWEN_ERROR_GENERIC;
-  }
-  GWEN_Buffer_AppendString(pbuf, DIRSEP "backends" DIRSEP);
-  GWEN_Buffer_AppendString(pbuf, name);
-  GWEN_Buffer_AppendString(pbuf, DIRSEP "settings.conf");
-
-  db=GWEN_DB_GetGroup(ab->data, GWEN_DB_FLAGS_DEFAULT,
-                      "banking/backends");
-  assert(db);
-  db=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, name);
-  assert(db);
-  DBG_INFO(AQBANKING_LOGDOMAIN,
-           "Reading file \"%s\"", GWEN_Buffer_GetStart(pbuf));
-  if (GWEN_DB_ReadFile(db, GWEN_Buffer_GetStart(pbuf),
-                       GWEN_DB_FLAGS_DEFAULT |
-                       GWEN_PATH_FLAGS_CREATE_GROUP |
-		       GWEN_DB_FLAGS_LOCKFILE, 0, 2000)) {
-    DBG_INFO(AQBANKING_LOGDOMAIN,
-	     "Old config file \"%s\", not found",
-	     GWEN_Buffer_GetStart(pbuf));
-    GWEN_Buffer_free(pbuf);
-    return 0;
-  }
-  GWEN_Buffer_free(pbuf);
-
-  /* sucessfully read */
-  return 0;
+int AB_Banking_InitProvider(AB_BANKING *ab, AB_PROVIDER *pro, uint32_t guiid) {
+  return AB_Provider_Init(pro, guiid);
 }
 
 
 
-GWEN_DB_NODE *AB_Banking_GetProviderData(AB_BANKING *ab,
-                                         AB_PROVIDER *pro) {
-  GWEN_DB_NODE *db;
-  GWEN_DB_NODE *dbT;
-
-  assert(ab);
-  assert(pro);
-
-  db=GWEN_DB_GetGroup(ab->data, GWEN_DB_FLAGS_DEFAULT,
-                      "banking/backends");
-  assert(db);
-  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-                       AB_Provider_GetEscapedName(pro));
-  if (!dbT && ab->lastVersion < ((1<<24) | (8<<16) | (1<<8) | 3)) {
-    int rv;
-
-    /* load separate file for older releases */
-    rv=AB_Banking__LoadOldProviderData(ab, AB_Provider_GetEscapedName(pro));
-    if (rv) {
-      DBG_ERROR(AQBANKING_LOGDOMAIN,
-                "Could not load settings for backend \"%s\"",
-                AB_Provider_GetName(pro));
-      abort();
-    }
-  }
-
-  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT,
-                       AB_Provider_GetEscapedName(pro));
-  assert(dbT);
-  return dbT;
-}
-
-
-
-int AB_Banking_InitProvider(AB_BANKING *ab, AB_PROVIDER *pro) {
-  return AB_Provider_Init(pro);
-}
-
-
-
-int AB_Banking_FiniProvider(AB_BANKING *ab, AB_PROVIDER *pro) {
+int AB_Banking_FiniProvider(AB_BANKING *ab, AB_PROVIDER *pro, uint32_t guiid) {
   int rv;
 
-  rv=AB_Provider_Fini(pro);
+  rv=AB_Provider_Fini(pro, guiid);
   if (rv) {
     DBG_INFO(AQBANKING_LOGDOMAIN, "Error deinit backend (%d)", rv);
   }
@@ -819,7 +651,7 @@ AB_PROVIDER *AB_Banking_GetProvider(AB_BANKING *ab, const char *name) {
     return pro;
   pro=AB_Banking__LoadProviderPlugin(ab, name);
   if (pro) {
-    if (AB_Banking_InitProvider(ab, pro)) {
+    if (AB_Banking_InitProvider(ab, pro, 0)) {
       DBG_ERROR(AQBANKING_LOGDOMAIN, "Could not init provider \"%s\"", name);
       AB_Provider_free(pro);
       return 0;
@@ -963,92 +795,5 @@ int AB_Banking_CheckCryptToken(AB_BANKING *ab,
 
 
 
-void AB_Banking__fillTransactionFromAccount(AB_TRANSACTION *t, const AB_ACCOUNT *a) {
-  const char *s;
-
-  s=AB_Transaction_GetLocalName(t);
-  if (!s)
-    AB_Transaction_SetLocalName(t, AB_Account_GetOwnerName(a));
-  s=AB_Transaction_GetLocalBankCode(t);
-  if (!s)
-    AB_Transaction_SetLocalBankCode(t, AB_Account_GetBankCode(a));
-  s=AB_Transaction_GetLocalAccountNumber(t);
-  if (!s)
-    AB_Transaction_SetLocalAccountNumber(t, AB_Account_GetAccountNumber(a));
-  s=AB_Transaction_GetLocalIban(t);
-  if (!s)
-    AB_Transaction_SetLocalIban(t, AB_Account_GetIBAN(a));
-  s=AB_Transaction_GetLocalBic(t);
-  if (!s)
-    AB_Transaction_SetLocalBic(t, AB_Account_GetBIC(a));
-}
-
-
-
-int AB_Banking_FillGapsInImExporterContext(AB_BANKING *ab, AB_IMEXPORTER_CONTEXT *iec) {
-  AB_IMEXPORTER_ACCOUNTINFO *iea;
-  int notFounds=0;
-
-  assert(iec);
-  iea=AB_ImExporterContext_GetFirstAccountInfo(iec);
-  while(iea) {
-    AB_ACCOUNT *a;
-
-    a=AB_Banking_GetAccountByCodeAndNumber(ab,
-					   AB_ImExporterAccountInfo_GetBankCode(iea),
-					   AB_ImExporterAccountInfo_GetAccountNumber(iea));
-    if (!a)
-      a=AB_Banking_GetAccountByIban(ab, AB_ImExporterAccountInfo_GetIban(iea));
-    if (a) {
-      AB_TRANSACTION *t;
-
-      AB_ImExporterAccountInfo_FillFromAccount(iea, a);
-
-      /* fill transactions */
-      t=AB_ImExporterAccountInfo_GetFirstTransaction(iea);
-      while(t) {
-	AB_Banking__fillTransactionFromAccount(t, a);
-	t=AB_ImExporterAccountInfo_GetNextTransaction(iea);
-      }
-
-      /* fill standing orders */
-      t=AB_ImExporterAccountInfo_GetFirstStandingOrder(iea);
-      while(t) {
-	AB_Banking__fillTransactionFromAccount(t, a);
-	t=AB_ImExporterAccountInfo_GetNextStandingOrder(iea);
-      }
-
-      /* fill transfers */
-      t=AB_ImExporterAccountInfo_GetFirstTransfer(iea);
-      while(t) {
-	AB_Banking__fillTransactionFromAccount(t, a);
-	t=AB_ImExporterAccountInfo_GetNextTransfer(iea);
-      }
-
-      /* fill dated transfers */
-      t=AB_ImExporterAccountInfo_GetFirstDatedTransfer(iea);
-      while(t) {
-	AB_Banking__fillTransactionFromAccount(t, a);
-	t=AB_ImExporterAccountInfo_GetNextDatedTransfer(iea);
-      }
-
-      /* fill noted transactions */
-      t=AB_ImExporterAccountInfo_GetFirstNotedTransaction(iea);
-      while(t) {
-	AB_Banking__fillTransactionFromAccount(t, a);
-	t=AB_ImExporterAccountInfo_GetNextNotedTransaction(iea);
-      }
-    }
-    else
-      notFounds++;
-
-    iea=AB_ImExporterContext_GetNextAccountInfo(iec);
-  }
-
-  return (notFounds==0)?0:1;
-}
-
-
-
 
 
