@@ -199,6 +199,39 @@ int AB_ImExporter_ImportBuffer(AB_IMEXPORTER *ie,
 
 
 
+int AB_ImExporter_ExportToBuffer(AB_IMEXPORTER *ie,
+				 AB_IMEXPORTER_CONTEXT *ctx,
+				 GWEN_BUFFER *buf,
+				 GWEN_DB_NODE *dbProfile,
+				 uint32_t guiid) {
+  int rv;
+  GWEN_IO_LAYER *io;
+
+  assert(ie);
+  assert(ctx);
+  assert(buf);
+  assert(dbProfile);
+
+  /* create io layer for this file (readonly) */
+  io=GWEN_Io_LayerMemory_new(buf);
+  assert(io);
+
+  rv=GWEN_Io_Manager_RegisterLayer(io);
+  if (rv<0) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Internal error: Could not register io layer (%d)", rv);
+    GWEN_Io_Layer_free(io);
+    return rv;
+  }
+
+  rv=AB_ImExporter_Export(ie, ctx, io, dbProfile, guiid);
+  GWEN_Io_Layer_DisconnectRecursively(io, NULL, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, 2000);
+  GWEN_Io_Layer_free(io);
+
+  return rv;
+}
+
+
+
 void AB_ImExporter_SetImportFn(AB_IMEXPORTER *ie,
                                AB_IMEXPORTER_IMPORT_FN f){
   assert(ie);
@@ -558,6 +591,7 @@ AB_ImExporterAccountInfo_dup(const AB_IMEXPORTER_ACCOUNTINFO *oi) {
   COPY_CHAR(description);
   iea->accountType=oi->accountType;
 #undef COPY_CHAR
+  iea->accountId=oi->accountId;
 
   iea->accStatusList=AB_AccountStatus_List_dup(oi->accStatusList);
   iea->transactions=AB_Transaction_List_dup(oi->transactions);
@@ -579,9 +613,8 @@ int AB_ImExporterAccountInfo_toDb(const AB_IMEXPORTER_ACCOUNTINFO *iea,
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, \
                          __STRING(NAME), iea->NAME)
 #define STORE_INT(NAME) \
-  if (iea->NAME) \
-    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, \
-                        __STRING(NAME), iea->NAME)
+  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, \
+                      __STRING(NAME), iea->NAME)
   STORE_CHAR(bankCode);
   STORE_CHAR(bankName);
   STORE_CHAR(accountNumber);
@@ -592,6 +625,7 @@ int AB_ImExporterAccountInfo_toDb(const AB_IMEXPORTER_ACCOUNTINFO *iea,
   STORE_CHAR(currency);
   STORE_CHAR(description);
   STORE_INT(accountType);
+  STORE_INT(accountId);
 #undef STORE_CHAR
 #undef STORE_INT
 
@@ -767,6 +801,7 @@ AB_ImExporterAccountInfo_fromDb(GWEN_DB_NODE *db){
   RESTORE_CHAR(currency);
   RESTORE_CHAR(description);
   RESTORE_INT(accountType, AB_AccountType_Bank);
+  RESTORE_INT(accountId, 0);
 #undef RESTORE_CHAR
 #undef RESTORE_INT
 
@@ -928,6 +963,14 @@ AB_ImExporterAccountInfo_TransactionsForEach(AB_IMEXPORTER_ACCOUNTINFO *iea,
 }
 
 
+
+int AB_ImExporterAccountInfo_GetTransactionCount(const AB_IMEXPORTER_ACCOUNTINFO *iea) {
+  assert(iea);
+  return AB_Transaction_List_GetCount(iea->transactions);
+}
+
+
+
 void AB_ImExporterAccountInfo_AddStandingOrder(AB_IMEXPORTER_ACCOUNTINFO *iea,
                                                AB_TRANSACTION *t){
   assert(iea);
@@ -970,6 +1013,13 @@ AB_ImExporterAccountInfo_GetNextStandingOrder(AB_IMEXPORTER_ACCOUNTINFO *iea){
 
 
 
+int AB_ImExporterAccountInfo_GetStandingOrderCount(const AB_IMEXPORTER_ACCOUNTINFO *iea) {
+  assert(iea);
+  return AB_Transaction_List_GetCount(iea->standingOrders);
+}
+
+
+
 void AB_ImExporterAccountInfo_AddDatedTransfer(AB_IMEXPORTER_ACCOUNTINFO *iea,
                                                AB_TRANSACTION *t){
   assert(iea);
@@ -1012,6 +1062,13 @@ AB_ImExporterAccountInfo_GetNextDatedTransfer(AB_IMEXPORTER_ACCOUNTINFO *iea){
 
 
 
+int AB_ImExporterAccountInfo_GetDatedTransferCount(const AB_IMEXPORTER_ACCOUNTINFO *iea) {
+  assert(iea);
+  return AB_Transaction_List_GetCount(iea->datedTransfers);
+}
+
+
+
 void AB_ImExporterAccountInfo_AddNotedTransaction(AB_IMEXPORTER_ACCOUNTINFO *iea,
                                                AB_TRANSACTION *t){
   assert(iea);
@@ -1054,6 +1111,13 @@ AB_ImExporterAccountInfo_GetNextNotedTransaction(AB_IMEXPORTER_ACCOUNTINFO *iea)
 
 
 
+int AB_ImExporterAccountInfo_GetNotedTransactionCount(const AB_IMEXPORTER_ACCOUNTINFO *iea) {
+  assert(iea);
+  return AB_Transaction_List_GetCount(iea->notedTransactions);
+}
+
+
+
 void AB_ImExporterAccountInfo_AddTransfer(AB_IMEXPORTER_ACCOUNTINFO *iea,
                                              AB_TRANSACTION *t){
   assert(iea);
@@ -1096,6 +1160,13 @@ AB_ImExporterAccountInfo_GetNextTransfer(AB_IMEXPORTER_ACCOUNTINFO *iea){
 
 
 
+int AB_ImExporterAccountInfo_GetTransferCount(const AB_IMEXPORTER_ACCOUNTINFO *iea) {
+  assert(iea);
+  return AB_Transaction_List_GetCount(iea->transfers);
+}
+
+
+
 const char*
 AB_ImExporterAccountInfo_GetBankCode(const AB_IMEXPORTER_ACCOUNTINFO *iea){
   assert(iea);
@@ -1273,6 +1344,20 @@ void AB_ImExporterAccountInfo_SetDescription(AB_IMEXPORTER_ACCOUNTINFO *iea,
 
 
 
+uint32_t AB_ImExporterAccountInfo_GetAccountId(const AB_IMEXPORTER_ACCOUNTINFO *iea) {
+  assert(iea);
+  return iea->accountId;
+}
+
+
+
+void AB_ImExporterAccountInfo_SetAccountId(AB_IMEXPORTER_ACCOUNTINFO *iea, uint32_t id) {
+  assert(iea);
+  iea->accountId=id;
+}
+
+
+
 void AB_ImExporterAccountInfo_AddAccountStatus(AB_IMEXPORTER_ACCOUNTINFO *iea,
                                              AB_ACCOUNT_STATUS *t){
   assert(iea);
@@ -1610,6 +1695,13 @@ AB_ImExporterContext_GetAccountInfo(AB_IMEXPORTER_CONTEXT *iec,
 
 
 
+int AB_ImExporterContext_GetAccountInfoCount(const AB_IMEXPORTER_CONTEXT *iec) {
+  assert(iec);
+  return AB_ImExporterAccountInfo_List_GetCount(iec->accountInfoList);
+}
+
+
+
 void AB_ImExporterContext_AddTransaction(AB_IMEXPORTER_CONTEXT *iec,
                                          AB_TRANSACTION *t){
   AB_IMEXPORTER_ACCOUNTINFO *iea;
@@ -2068,8 +2160,7 @@ void GWENHYWFAR_CB AB_Plugin_ImExporter_FreeData(void *bp, void *p) {
 
 
 AB_IMEXPORTER *AB_Plugin_ImExporter_Factory(GWEN_PLUGIN *pl,
-					    AB_BANKING *ab,
-					    GWEN_DB_NODE *db) {
+					    AB_BANKING *ab) {
   AB_PLUGIN_IMEXPORTER *xpl;
 
   assert(pl);
@@ -2077,7 +2168,7 @@ AB_IMEXPORTER *AB_Plugin_ImExporter_Factory(GWEN_PLUGIN *pl,
   assert(xpl);
 
   assert(xpl->pluginFactoryFn);
-  return xpl->pluginFactoryFn(pl, ab, db);
+  return xpl->pluginFactoryFn(pl, ab);
 }
 
 
