@@ -225,15 +225,12 @@ int AH_Job_MultiTransfer__ValidateTransfer(AB_JOB *bj,
   n=0;
   if (sl) {
     GWEN_STRINGLISTENTRY *se;
-    GWEN_STRINGLIST *nsl;
     const char *p;
 
-    nsl=GWEN_StringList_new();
     se=GWEN_StringList_FirstEntry(sl);
     while(se) {
       p=GWEN_StringListEntry_Data(se);
       if (p && *p) {
-	char *np;
 	int l;
 	GWEN_BUFFER *tbuf;
 
@@ -241,27 +238,20 @@ int AH_Job_MultiTransfer__ValidateTransfer(AB_JOB *bj,
 	if (maxn && n>maxn) {
 	  DBG_ERROR(AQHBCI_LOGDOMAIN,
 		    "Too many purpose lines (%d>%d)", n, maxn);
-	  GWEN_StringList_free(nsl);
 	  return GWEN_ERROR_INVALID;
 	}
 	tbuf=GWEN_Buffer_new(0, maxs, 0, 1);
 	AB_ImExporter_Utf8ToDta(p, -1, tbuf);
 	l=GWEN_Buffer_GetUsedBytes(tbuf);
+	GWEN_Buffer_free(tbuf);
 	if (l>maxs) {
 	  DBG_ERROR(AQHBCI_LOGDOMAIN,
 		    "Too many chars in purpose line %d (%d>%d)", n, l, maxs);
-	  GWEN_StringList_free(nsl);
 	  return GWEN_ERROR_INVALID;
 	}
-	np=(char*)GWEN_Memory_malloc(l+1);
-	memmove(np, GWEN_Buffer_GetStart(tbuf), l+1);
-	GWEN_Buffer_free(tbuf);
-	/* let string list take the newly alllocated string */
-	GWEN_StringList_AppendString(nsl, np, 1, 0);
       }
       se=GWEN_StringListEntry_Next(se);
     } /* while */
-    AB_Transaction_SetPurpose(t, nsl);
   }
   if (!n) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "No purpose lines");
@@ -281,15 +271,12 @@ int AH_Job_MultiTransfer__ValidateTransfer(AB_JOB *bj,
   n=0;
   if (sl) {
     GWEN_STRINGLISTENTRY *se;
-    GWEN_STRINGLIST *nsl;
     const char *p;
 
-    nsl=GWEN_StringList_new();
     se=GWEN_StringList_FirstEntry(sl);
     while(se) {
       p=GWEN_StringListEntry_Data(se);
       if (p && *p) {
-	char *np;
 	int l;
         GWEN_BUFFER *tbuf;
 
@@ -298,28 +285,21 @@ int AH_Job_MultiTransfer__ValidateTransfer(AB_JOB *bj,
 	  DBG_ERROR(AQHBCI_LOGDOMAIN,
 		    "Too many remote name lines (%d>%d)",
 		    n, maxn);
-          GWEN_StringList_free(nsl);
 	  return GWEN_ERROR_INVALID;
 	}
 	tbuf=GWEN_Buffer_new(0, maxs, 0, 1);
         AB_ImExporter_Utf8ToDta(p, -1, tbuf);
 	l=GWEN_Buffer_GetUsedBytes(tbuf);
+	GWEN_Buffer_free(tbuf);
 	if (l>maxs) {
 	  DBG_ERROR(AQHBCI_LOGDOMAIN,
 		   "Too many chars in remote name line %d (%d>%d)",
 		   n, l, maxs);
-          GWEN_StringList_free(nsl);
 	  return GWEN_ERROR_INVALID;
 	}
-	np=(char*)GWEN_Memory_malloc(l+1);
-	memmove(np, GWEN_Buffer_GetStart(tbuf), l+1);
-	GWEN_Buffer_free(tbuf);
-	/* let string list take the newly alllocated string */
-	GWEN_StringList_AppendString(nsl, np, 1, 0);
       }
       se=GWEN_StringListEntry_Next(se);
     } /* while */
-    AB_Transaction_SetRemoteName(t, nsl);
   }
   if (!n) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "No remote name lines");
@@ -483,7 +463,9 @@ int AH_Job_MultiTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
   case AH_Job_ExchangeModeArgs: {
     GWEN_DB_NODE *dbArgs;
     const AB_TRANSACTION *ot;
+    char groupIdBuf[10];
 
+    groupIdBuf[0]=0;
     dbArgs=AH_Job_GetArguments(j);
     assert(dbArgs);
     if (aj->isTransfer)
@@ -502,6 +484,13 @@ int AH_Job_MultiTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
 	AB_Job_SetStatus(bj, AB_Job_StatusError);
 	return GWEN_ERROR_INVALID;
       }
+      AB_Transaction_SetGroupId(t, AH_Job_GetId(j));
+      DBG_ERROR(0, "Setting groupID to %d", AH_Job_GetId(j));
+      if (groupIdBuf[0]==0) {
+	snprintf(groupIdBuf, sizeof(groupIdBuf)-1, "%08d", AH_Job_GetId(j));
+        groupIdBuf[sizeof(groupIdBuf)-1]=0;
+      }
+
       /* store the validated transaction back into application job,
        * to allow the application to recognize answers to this job later */
       if (aj->isTransfer)
@@ -512,6 +501,11 @@ int AH_Job_MultiTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
       dbT=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT,
                            "transfers");
       assert(dbT);
+      /* store customer reference */
+      GWEN_DB_SetCharValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			   "custref",
+			   groupIdBuf);
+
       dbT=GWEN_DB_GetGroup(dbT, GWEN_DB_FLAGS_DEFAULT,
 			   "transactions");
       assert(dbT);
