@@ -53,6 +53,7 @@ AH_JOB *AH_Job_GetKeys_new(AB_USER *u){
   AH_JOB *j;
   AH_JOB_GETKEYS *jd;
   GWEN_DB_NODE *args;
+  int version;
 
   assert(u);
   j=AH_Job_new("JobGetKeys", u, 0);
@@ -81,7 +82,19 @@ AH_JOB *AH_Job_GetKeys_new(AB_USER *u){
                       0);
   GWEN_DB_SetCharValue(args, GWEN_DB_FLAGS_OVERWRITE_VARS,
                        "open/ident/systemId",
-                       "0");
+		       "0");
+
+  version=AH_User_GetRdhType(u);
+  if (version==0)
+    version=1;
+  GWEN_DB_SetCharValue(args, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		       "open/cryptKey/secprofile/code", "RDH");
+  GWEN_DB_SetIntValue(args, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "open/cryptKey/secprofile/version", version);
+  GWEN_DB_SetCharValue(args, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		       "open/signKey/secprofile/code", "RDH");
+  GWEN_DB_SetIntValue(args, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "open/signKey/secprofile/version", version);
 
   return j;
 }
@@ -165,7 +178,7 @@ int AH_Job_GetKeys_Process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx,
                                    "data/GetKeyResponse");
     if (dbKeyResponse) {
       unsigned int bs;
-      const void *p;
+      const uint8_t *p;
 
       DBG_DEBUG(AQHBCI_LOGDOMAIN, "Got this key response:");
       if (GWEN_Logger_GetLevel(AQHBCI_LOGDOMAIN)>=GWEN_LoggerLevel_Debug)
@@ -184,15 +197,18 @@ int AH_Job_GetKeys_Process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx,
 	int keySize;
 	uint32_t flags=0;
 
-        /* calculate key size in bytes */
-	if (bs>256)       /* >2048 bits? */
-	  keySize=512;    /*  -> 4096 */
-	else if (bs>128)  /* >1024 bits? */
-	  keySize=256;    /*  -> 2048 */
-	else if (bs>96)   /* >768 bits? */
-	  keySize=128;    /*  -> 1024 bits */
-	else
-	  keySize=96;     /* otherwise 768 bits */
+	/* skip zero bytes if any */
+	while(bs && *p==0) {
+	  p++;
+	  bs--;
+	}
+
+	/* calculate key size in bytes */
+	if (bs<=96)
+	  keySize=96;
+	else {
+	  keySize=bs;
+	}
 
 	s=GWEN_DB_GetCharValue(dbKeyResponse,
 			       "keyname/keytype", 0,
@@ -318,31 +334,39 @@ AH_JOB *AH_Job_SendKeys_new(AB_USER *u,
 			    const GWEN_CRYPT_TOKEN_KEYINFO *signKeyInfo,
 			    const GWEN_CRYPT_TOKEN_KEYINFO *authKeyInfo) {
   AH_JOB *j;
+  GWEN_DB_NODE *dbArgs;
   GWEN_DB_NODE *dbKey;
+  int version;
 
   assert(u);
 
-  j=AH_Job_new("JobSendKeys", u, 0);
+  if (authKeyInfo)
+    j=AH_Job_new("JobSendKeysWithAuthKey", u, 0);
+  else
+    j=AH_Job_new("JobSendKeys", u, 0);
   if (!j) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "JobSendKeys not supported, should not happen");
     return NULL;
   }
 
   /* set arguments */
-  dbKey=GWEN_DB_GetGroup(AH_Job_GetArguments(j),
-                         GWEN_DB_FLAGS_DEFAULT,
-                         "cryptKey");
+  dbArgs=AH_Job_GetArguments(j);
+  dbKey=GWEN_DB_GetGroup(dbArgs,
+			 GWEN_DB_FLAGS_DEFAULT,
+			 "cryptKey");
   assert(dbKey);
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Preparing crypt key");
   if (AH_Job_SendKeys_PrepareKey(j, dbKey, cryptKeyInfo, 0)) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not prepare cryptkey");
     AH_Job_free(j);
     return 0;
   }
 
-  dbKey=GWEN_DB_GetGroup(AH_Job_GetArguments(j),
+  dbKey=GWEN_DB_GetGroup(dbArgs,
                          GWEN_DB_FLAGS_DEFAULT,
                          "signKey");
   assert(dbKey);
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Preparing sign key");
   if (AH_Job_SendKeys_PrepareKey(j, dbKey, signKeyInfo, 1)) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not prepare signkey");
     AH_Job_free(j);
@@ -350,16 +374,37 @@ AH_JOB *AH_Job_SendKeys_new(AB_USER *u,
   }
 
   if (authKeyInfo) {
-    dbKey=GWEN_DB_GetGroup(AH_Job_GetArguments(j),
+    dbKey=GWEN_DB_GetGroup(dbArgs,
 			   GWEN_DB_FLAGS_DEFAULT,
 			   "authKey");
     assert(dbKey);
+    DBG_INFO(AQHBCI_LOGDOMAIN, "Preparing auth key");
     if (AH_Job_SendKeys_PrepareKey(j, dbKey, authKeyInfo, 2)) {
       DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not prepare authkey");
       AH_Job_free(j);
       return 0;
     }
   }
+  else {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "No auth key info");
+  }
+
+  version=AH_User_GetRdhType(u);
+  if (version==0)
+    version=1;
+  GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		       "cryptKey/secprofile/code", "RDH");
+  GWEN_DB_SetIntValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "cryptKey/secprofile/version", version);
+
+  GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		       "signKey/secprofile/code", "RDH");
+  GWEN_DB_SetIntValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "signKey/secprofile/version", version);
+  GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		       "authKey/secprofile/code", "RDH");
+  GWEN_DB_SetIntValue(dbArgs, GWEN_DB_FLAGS_OVERWRITE_VARS,
+		      "authKey/secprofile/version", version);
 
   DBG_INFO(AQHBCI_LOGDOMAIN, "JobSendKeys created");
   return j;
@@ -431,17 +476,54 @@ int AH_Job_SendKeys_PrepareKey(AH_JOB *j,
   /* set key */
   if (kn==0) { /* crypt key */
     GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
-                        "key/purpose", 5);
-
+			"key/purpose", 5);
+    switch(AH_User_GetRdhType(u)) {
+    case 10:
+      GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			  "key/opmode", 2);
+      break;
+    case 5:
+      GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			  "key/opmode", 18);
+      break;
+    case 2:
+      GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			  "key/opmode", 2);
+      break;
+    case 0:
+    case 1:
+    default:
+      GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			  "key/opmode", 2);
+      break;
+    }
   }
-  else { /* sign key */
+  else {
+    /* sign key */
     GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
-                        "key/purpose", 6);
+			"key/purpose", 6);
+    switch(AH_User_GetRdhType(u)) {
+    case 10:
+      GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			  "key/opmode", 19);
+      break;
+    case 5:
+      GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			  "key/opmode", 18);
+      break;
+    case 2:
+      GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			  "key/opmode", 17);
+      break;
+    case 0:
+    case 1:
+    default:
+      GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			  "key/opmode", 16);
+      break;
+    }
   }
-  /* TODO: set purpose for auth key */
 
-  GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
-                      "key/opmode", 16);
   GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "key/type", 10);
   GWEN_DB_SetIntValue(dbKey, GWEN_DB_FLAGS_OVERWRITE_VARS,
@@ -1384,10 +1466,10 @@ int AH_Job_Tan_FinishSetup(AH_JOB *j) {
 
     v=AH_Job_GetChallengeValue(j);
     if (!v) {
-      DBG_ERROR(AQHBCI_LOGDOMAIN, "Missing challenge value");
-      return GWEN_ERROR_GENERIC;
+      DBG_INFO(AQHBCI_LOGDOMAIN,
+	       "Missing challenge amount value, ignoring");
+      /*return GWEN_ERROR_GENERIC; */
     }
-  
     else {
       GWEN_BUFFER *btmp;
   
@@ -1408,7 +1490,7 @@ int AH_Job_Tan_FinishSetup(AH_JOB *j) {
     }
   }
   else {
-    DBG_ERROR(0, "Challenge amount not needed");
+    DBG_INFO(AQHBCI_LOGDOMAIN, "Challenge amount not needed");
   }
 
   return 0;
@@ -1573,6 +1655,9 @@ AH_JOB *AH_Job_ChangePin_new(AB_USER *u, const char *newPin){
     return 0;
   }
 
+  /* set challenge class */
+  AH_Job_SetChallengeClass(j, 90);
+
   /* set arguments */
   dbArgs=AH_Job_GetArguments(j);
   assert(dbArgs);
