@@ -63,8 +63,6 @@ AH_OUTBOX__CBOX *AH_Outbox__CBox_new(AH_HBCI *hbci,
   cbox->hbci=hbci;
   cbox->outbox=ob;
 
-  cbox->guiid=ob->guiid;
-
   return cbox;
 }
 
@@ -268,6 +266,7 @@ void AH_Outbox__CBox_Finish(AH_OUTBOX__CBOX *cbox){
 
   assert(cbox);
 
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Finishing customer box");
   while((jq=AH_JobQueue_List_First(cbox->finishedQueues))) {
     AH_JOB_LIST *jl;
     AH_JOB *j;
@@ -299,6 +298,18 @@ void AH_Outbox__CBox_Finish(AH_OUTBOX__CBOX *cbox){
     AH_JobQueue_free(jq);
   } /* while */
 
+  if (AH_Job_List_GetCount(cbox->todoJobs)) {
+    AH_JOB *j;
+
+    while((j=AH_Job_List_First(cbox->todoJobs))) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Moving job \"%s\" from todo queue to finished jobs",
+               AH_Job_GetName(j));
+      AH_Job_List_Del(j);
+      AH_Job_List_Add(j, cbox->finishedJobs);
+    } /* while */
+  }
+
+
   /* check for results for pending jobs */
   AH_Outbox__CBox_CheckPending(cbox);
 }
@@ -407,7 +418,7 @@ GWEN_TIME *AH_Outbox__CBox_GetLatestPendingDate(AH_OUTBOX__CBOX *cbox) {
 
 
 
-int AH_Outbox__CBox_Prepare(AH_OUTBOX__CBOX *cbox){
+int AH_Outbox__CBox_Prepare(AH_OUTBOX__CBOX *cbox, uint32_t guiid){
   AH_JOB *j;
   unsigned int errors;
   AH_JOBQUEUE *jq;
@@ -426,7 +437,7 @@ int AH_Outbox__CBox_Prepare(AH_OUTBOX__CBOX *cbox){
     next=AH_Job_List_Next(j);
     st=AH_Job_GetStatus(j);
     if (st==AH_JobStatusToDo) {
-      int rv=AH_Job_Prepare(j, cbox->guiid);
+      int rv=AH_Job_Prepare(j, guiid);
       if (rv<0 && rv!=GWEN_ERROR_NOT_SUPPORTED) {
 	DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
 	AH_Job_SetStatus(j, AH_JobStatusError);
@@ -678,31 +689,32 @@ int AH_Outbox__CBox_Prepare(AH_OUTBOX__CBOX *cbox){
 
 int AH_Outbox__CBox_SendQueue(AH_OUTBOX__CBOX *cbox, int timeout,
 			      AH_DIALOG *dlg,
-			      AH_JOBQUEUE *jq){
+			      AH_JOBQUEUE *jq,
+			      uint32_t guiid){
   AH_MSG *msg;
   int rv;
 
   DBG_NOTICE(AQHBCI_LOGDOMAIN, "Encoding queue");
-  GWEN_Gui_ProgressLog(cbox->guiid,
+  GWEN_Gui_ProgressLog(guiid,
 		       GWEN_LoggerLevel_Info,
 		       I18N("Encoding queue"));
   msg=AH_JobQueue_ToMessage(jq, dlg);
   if (!msg) {
     DBG_INFO(AQHBCI_LOGDOMAIN, "Could not encode queue");
-    GWEN_Gui_ProgressLog(cbox->guiid,
+    GWEN_Gui_ProgressLog(guiid,
 			 GWEN_LoggerLevel_Error,
 			 I18N("Unable to encode"));
     return GWEN_ERROR_GENERIC;
   }
   DBG_NOTICE(AQHBCI_LOGDOMAIN, "Sending queue");
-  GWEN_Gui_ProgressLog(cbox->guiid,
+  GWEN_Gui_ProgressLog(guiid,
 		       GWEN_LoggerLevel_Info,
 		       I18N("Sending queue"));
 
   rv=AH_Dialog_SendMessage(dlg, msg);
   if (rv) {
     DBG_NOTICE(AQHBCI_LOGDOMAIN, "Could not send message");
-    GWEN_Gui_ProgressLog(cbox->guiid,
+    GWEN_Gui_ProgressLog(guiid,
 			 GWEN_LoggerLevel_Error,
 			 I18N("Unable to send (network error)"));
     return rv;
@@ -716,14 +728,15 @@ int AH_Outbox__CBox_SendQueue(AH_OUTBOX__CBOX *cbox, int timeout,
 int AH_Outbox__CBox_RecvQueue(AH_OUTBOX__CBOX *cbox,
 			      int timeout,
 			      AH_DIALOG *dlg,
-			      AH_JOBQUEUE *jq){
+			      AH_JOBQUEUE *jq,
+			      uint32_t guiid){
   AH_MSG *msg;
   GWEN_DB_NODE *rsp;
   int rv;
 
   assert(cbox);
 
-  GWEN_Gui_ProgressLog(cbox->guiid,
+  GWEN_Gui_ProgressLog(guiid,
 		       GWEN_LoggerLevel_Info,
 		       I18N("Waiting for response"));
 
@@ -731,14 +744,14 @@ int AH_Outbox__CBox_RecvQueue(AH_OUTBOX__CBOX *cbox,
   if (rv<0) {
     DBG_INFO(AQHBCI_LOGDOMAIN,
 	     "No message within specified timeout, giving up");
-    GWEN_Gui_ProgressLog(cbox->guiid,
+    GWEN_Gui_ProgressLog(guiid,
 			 GWEN_LoggerLevel_Error,
 			 I18N("No response (timeout)"));
     return AB_ERROR_NETWORK;
   }
 
   DBG_INFO(AQHBCI_LOGDOMAIN, "Got a message");
-  GWEN_Gui_ProgressLog(cbox->guiid,
+  GWEN_Gui_ProgressLog(guiid,
 		       GWEN_LoggerLevel_Info,
 		       I18N("Response received"));
 
@@ -748,7 +761,7 @@ int AH_Outbox__CBox_RecvQueue(AH_OUTBOX__CBOX *cbox,
     DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not decode this message:");
     AH_Msg_Dump(msg, stderr, 2);
     GWEN_DB_Group_free(rsp);
-    GWEN_Gui_ProgressLog(cbox->guiid,
+    GWEN_Gui_ProgressLog(guiid,
 			 GWEN_LoggerLevel_Error,
 			 I18N("Bad response (unable to decode)"));
     return GWEN_ERROR_GENERIC;
@@ -764,7 +777,7 @@ int AH_Outbox__CBox_RecvQueue(AH_OUTBOX__CBOX *cbox,
     GWEN_DB_Dump(rsp, stderr, 2);
     GWEN_DB_Group_free(rsp);
     AH_Msg_free(msg);
-    GWEN_Gui_ProgressLog(cbox->guiid,
+    GWEN_Gui_ProgressLog(guiid,
 			 GWEN_LoggerLevel_Error,
 			 I18N("Bad response (bad message reference)"));
     return GWEN_ERROR_GENERIC;
@@ -774,13 +787,13 @@ int AH_Outbox__CBox_RecvQueue(AH_OUTBOX__CBOX *cbox,
   if (rv) {
     if (rv==GWEN_ERROR_ABORTED) {
       DBG_INFO(AQHBCI_LOGDOMAIN, "Dialog aborted by server");
-      GWEN_Gui_ProgressLog(cbox->guiid,
+      GWEN_Gui_ProgressLog(guiid,
 			   GWEN_LoggerLevel_Error,
 			   I18N("Dialog aborted by server"));
     }
     else {
       DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not dispatch response");
-      GWEN_Gui_ProgressLog(cbox->guiid,
+      GWEN_Gui_ProgressLog(guiid,
 			   GWEN_LoggerLevel_Error,
 			   I18N("Bad response (unable to dispatch)"));
     }
@@ -800,14 +813,15 @@ int AH_Outbox__CBox_RecvQueue(AH_OUTBOX__CBOX *cbox,
 int AH_Outbox__CBox_SendAndRecvQueue(AH_OUTBOX__CBOX *cbox,
 				     int timeout,
 				     AH_DIALOG *dlg,
-				     AH_JOBQUEUE *jq){
+				     AH_JOBQUEUE *jq,
+				     uint32_t guiid){
   int rv;
 
   if ((AH_JobQueue_GetFlags(jq) & AH_JOBQUEUE_FLAGS_NEEDTAN) &&
       AH_Dialog_GetItanProcessType(dlg)!=0) {
     DBG_DEBUG(AQHBCI_LOGDOMAIN, "iTAN mode");
 
-    rv=AH_Outbox__CBox_Itan(cbox, dlg, jq, timeout);
+    rv=AH_Outbox__CBox_Itan(cbox, dlg, jq, timeout, guiid);
     if (rv) {
       DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
       return rv;
@@ -815,7 +829,7 @@ int AH_Outbox__CBox_SendAndRecvQueue(AH_OUTBOX__CBOX *cbox,
   }
   else {
     DBG_DEBUG(AQHBCI_LOGDOMAIN, "Normal mode");
-    rv=AH_Outbox__CBox_SendQueue(cbox, timeout, dlg, jq);
+    rv=AH_Outbox__CBox_SendQueue(cbox, timeout, dlg, jq, guiid);
     if (rv) {
       DBG_INFO(AQHBCI_LOGDOMAIN, "Error sending queue");
       return rv;
@@ -824,7 +838,7 @@ int AH_Outbox__CBox_SendAndRecvQueue(AH_OUTBOX__CBOX *cbox,
     AH_JobQueue_SetJobStatusOnMatch(jq, AH_JobStatusEncoded,
                                     AH_JobStatusSent);
 
-    rv=AH_Outbox__CBox_RecvQueue(cbox, timeout, dlg, jq);
+    rv=AH_Outbox__CBox_RecvQueue(cbox, timeout, dlg, jq, guiid);
     if (rv) {
       DBG_INFO(AQHBCI_LOGDOMAIN, "Error receiving queue response");
       return rv;
@@ -839,7 +853,8 @@ int AH_Outbox__CBox_SendAndRecvQueue(AH_OUTBOX__CBOX *cbox,
 
 int AH_Outbox__CBox_OpenDialog(AH_OUTBOX__CBOX *cbox, int timeout,
 			       AH_DIALOG *dlg,
-			       uint32_t jqFlags) {
+			       uint32_t jqFlags,
+			       uint32_t guiid) {
   AH_JOBQUEUE *jqDlgOpen;
   AH_JOB *jDlgOpen;
   int rv;
@@ -887,7 +902,7 @@ int AH_Outbox__CBox_OpenDialog(AH_OUTBOX__CBOX *cbox, int timeout,
     AH_Dialog_AddFlags(dlg, AH_DIALOG_FLAGS_ANONYMOUS);
   }
 
-  GWEN_Gui_ProgressLog(cbox->guiid,
+  GWEN_Gui_ProgressLog(guiid,
 		       GWEN_LoggerLevel_Notice,
 		       I18N("Opening dialog"));
   jqDlgOpen=AH_JobQueue_new(cbox->user);
@@ -901,7 +916,7 @@ int AH_Outbox__CBox_OpenDialog(AH_OUTBOX__CBOX *cbox, int timeout,
     return GWEN_ERROR_GENERIC;
   }
 
-  rv=AH_Outbox__CBox_SendAndRecvQueue(cbox, timeout, dlg, jqDlgOpen);
+  rv=AH_Outbox__CBox_SendAndRecvQueue(cbox, timeout, dlg, jqDlgOpen, guiid);
   if (rv) {
     DBG_NOTICE(AQHBCI_LOGDOMAIN, "Could not exchange message");
     AH_JobQueue_free(jqDlgOpen);
@@ -912,12 +927,12 @@ int AH_Outbox__CBox_OpenDialog(AH_OUTBOX__CBOX *cbox, int timeout,
     if (AH_Job_HasItanResult(jDlgOpen)) {
       DBG_NOTICE(AQHBCI_LOGDOMAIN,
 		 "Adjusting to iTAN modes of the server");
-      GWEN_Gui_ProgressLog(cbox->guiid,
+      GWEN_Gui_ProgressLog(guiid,
 			   GWEN_LoggerLevel_Notice,
 			   I18N("Adjusting to iTAN modes of the server"));
       /* do not call AH_Job_CommitSystemData() here, the iTAN modes have already
        * been caught by AH_JobQueue_DispatchMessage()
-        AH_Job_CommitSystemData(jDlgOpen, cbox->guiid); */
+        AH_Job_CommitSystemData(jDlgOpen, guiid); */
       AH_JobQueue_free(jqDlgOpen);
       return 1;
     }
@@ -928,7 +943,7 @@ int AH_Outbox__CBox_OpenDialog(AH_OUTBOX__CBOX *cbox, int timeout,
     }
   }
   DBG_NOTICE(AQHBCI_LOGDOMAIN, "Dialog open request done");
-  rv=AH_Job_CommitSystemData(jDlgOpen, cbox->guiid);
+  rv=AH_Job_CommitSystemData(jDlgOpen, guiid);
   AH_JobQueue_free(jqDlgOpen);
   return rv;
 }
@@ -938,14 +953,15 @@ int AH_Outbox__CBox_OpenDialog(AH_OUTBOX__CBOX *cbox, int timeout,
 int AH_Outbox__CBox_CloseDialog(AH_OUTBOX__CBOX *cbox,
 				int timeout,
 				AH_DIALOG *dlg,
-				uint32_t jqFlags) {
+				uint32_t jqFlags,
+				uint32_t guiid) {
   AH_JOBQUEUE *jqDlgClose;
   AH_JOB *jDlgClose;
   GWEN_DB_NODE *db;
   uint32_t dlgFlags;
   int rv;
 
-  GWEN_Gui_ProgressLog(cbox->guiid,
+  GWEN_Gui_ProgressLog(guiid,
 		       GWEN_LoggerLevel_Notice,
 		       I18N("Closing dialog"));
   DBG_NOTICE(AQHBCI_LOGDOMAIN, "Sending dialog close request (flags=%08x)",
@@ -983,14 +999,14 @@ int AH_Outbox__CBox_CloseDialog(AH_OUTBOX__CBOX *cbox,
     return GWEN_ERROR_GENERIC;
   }
 
-  rv=AH_Outbox__CBox_SendAndRecvQueue(cbox, timeout, dlg, jqDlgClose);
+  rv=AH_Outbox__CBox_SendAndRecvQueue(cbox, timeout, dlg, jqDlgClose, guiid);
   if (rv) {
     DBG_NOTICE(AQHBCI_LOGDOMAIN, "Could not exchange message");
     AH_JobQueue_free(jqDlgClose);
     return rv;
   }
   DBG_NOTICE(AQHBCI_LOGDOMAIN, "Dialog closed");
-  rv=AH_Job_CommitSystemData(jDlgClose, cbox->guiid);
+  rv=AH_Job_CommitSystemData(jDlgClose, guiid);
   AH_JobQueue_free(jqDlgClose);
   if (rv) {
     DBG_NOTICE(AQHBCI_LOGDOMAIN, "Could not commit system data");
@@ -1030,7 +1046,8 @@ void AH_Outbox__CBox_HandleQueueError(AH_OUTBOX__CBOX *cbox,
 int AH_Outbox__CBox_PerformQueue(AH_OUTBOX__CBOX *cbox,
 				 AH_DIALOG *dlg,
 				 AH_JOBQUEUE *jq,
-				 int timeout) {
+				 int timeout,
+				 uint32_t guiid) {
   int jobsTodo;
   int rv;
 
@@ -1125,7 +1142,7 @@ int AH_Outbox__CBox_PerformQueue(AH_OUTBOX__CBOX *cbox,
       break;
 
     /* jq now contains all jobs to be executed */
-    rv=AH_Outbox__CBox_SendAndRecvQueue(cbox, timeout, dlg, jq);
+    rv=AH_Outbox__CBox_SendAndRecvQueue(cbox, timeout, dlg, jq, guiid);
     if (rv) {
       AH_Outbox__CBox_HandleQueueError(cbox, jq,
                                        "Error performing queue");
@@ -1141,7 +1158,8 @@ int AH_Outbox__CBox_PerformQueue(AH_OUTBOX__CBOX *cbox,
 
 int AH_Outbox__CBox_PerformNonDialogQueues(AH_OUTBOX__CBOX *cbox,
 					   int timeout,
-					   AH_JOBQUEUE_LIST *jql){
+					   AH_JOBQUEUE_LIST *jql,
+					   uint32_t guiid){
   AH_DIALOG *dlg;
   AH_JOBQUEUE *jq;
   int rv=0;
@@ -1155,7 +1173,7 @@ int AH_Outbox__CBox_PerformNonDialogQueues(AH_OUTBOX__CBOX *cbox,
   }
 
   for(i=0; i<2; i++) {
-    dlg=AH_Dialog_new(cbox->user, cbox->guiid);
+    dlg=AH_Dialog_new(cbox->user, guiid);
     rv=AH_Dialog_Connect(dlg, AH_HBCI_GetConnectTimeout(cbox->hbci));
     if (rv) {
       DBG_INFO(AQHBCI_LOGDOMAIN,
@@ -1173,7 +1191,7 @@ int AH_Outbox__CBox_PerformNonDialogQueues(AH_OUTBOX__CBOX *cbox,
     jqflags=AH_JobQueue_GetFlags(jq);
   
     /* open dialog */
-    rv=AH_Outbox__CBox_OpenDialog(cbox, timeout, dlg, jqflags);
+    rv=AH_Outbox__CBox_OpenDialog(cbox, timeout, dlg, jqflags, guiid);
     if (rv==0)
       break;
     else if (rv<0) {
@@ -1207,7 +1225,7 @@ int AH_Outbox__CBox_PerformNonDialogQueues(AH_OUTBOX__CBOX *cbox,
   rv=0;
   while((jq=AH_JobQueue_List_First(jql))) {
     AH_JobQueue_List_Del(jq);
-    rv=AH_Outbox__CBox_PerformQueue(cbox, dlg, jq, timeout);
+    rv=AH_Outbox__CBox_PerformQueue(cbox, dlg, jq, timeout, guiid);
     if (rv)
       break;
   } /* while */
@@ -1222,7 +1240,7 @@ int AH_Outbox__CBox_PerformNonDialogQueues(AH_OUTBOX__CBOX *cbox,
   }
 
   /* close dialog */
-  rv=AH_Outbox__CBox_CloseDialog(cbox, timeout, dlg, jqflags);
+  rv=AH_Outbox__CBox_CloseDialog(cbox, timeout, dlg, jqflags, guiid);
   if (rv) {
     DBG_INFO(AQHBCI_LOGDOMAIN,
              "Could not close dialog, ignoring");
@@ -1242,12 +1260,13 @@ int AH_Outbox__CBox_PerformNonDialogQueues(AH_OUTBOX__CBOX *cbox,
 
 int AH_Outbox__CBox_PerformDialogQueue(AH_OUTBOX__CBOX *cbox,
 				       int timeout,
-				       AH_JOBQUEUE *jq){
+				       AH_JOBQUEUE *jq,
+				       uint32_t guiid){
   AH_DIALOG *dlg;
   int rv;
 
   /* open connection */
-  dlg=AH_Dialog_new(cbox->user, cbox->guiid);
+  dlg=AH_Dialog_new(cbox->user, guiid);
   rv=AH_Dialog_Connect(dlg, AH_HBCI_GetConnectTimeout(cbox->hbci));
   if (rv) {
     DBG_INFO(AQHBCI_LOGDOMAIN,
@@ -1272,7 +1291,7 @@ int AH_Outbox__CBox_PerformDialogQueue(AH_OUTBOX__CBOX *cbox,
   }
 
   /* handle queue */
-  rv=AH_Outbox__CBox_PerformQueue(cbox, dlg, jq, timeout);
+  rv=AH_Outbox__CBox_PerformQueue(cbox, dlg, jq, timeout, guiid);
   if (rv) {
     AH_Dialog_Disconnect(dlg, 2);
     AH_Dialog_free(dlg);
@@ -1326,7 +1345,8 @@ void AH_Outbox__CBox_HandleQueueListError(AH_OUTBOX__CBOX *cbox,
 
 
 int AH_Outbox__CBox_SendAndRecvDialogQueues(AH_OUTBOX__CBOX *cbox,
-					    int timeout) {
+					    int timeout,
+					    uint32_t guiid) {
   AH_JOBQUEUE_LIST *jqlWanted;
   AH_JOBQUEUE_LIST *jqlRest;
   int rv;
@@ -1345,7 +1365,7 @@ int AH_Outbox__CBox_SendAndRecvDialogQueues(AH_OUTBOX__CBOX *cbox,
 
     /* there are matching queues, handle them */
     while((jq=AH_JobQueue_List_First(jqlWanted))) {
-      rv=AH_Outbox__CBox_PerformDialogQueue(cbox, timeout, jq);
+      rv=AH_Outbox__CBox_PerformDialogQueue(cbox, timeout, jq, guiid);
       if (rv) {
 	DBG_ERROR(AQHBCI_LOGDOMAIN,
 		  "Error performing queue (%d)", rv);
@@ -1369,7 +1389,8 @@ int AH_Outbox__CBox_SendAndRecvDialogQueues(AH_OUTBOX__CBOX *cbox,
 int AH_Outbox__CBox_SendAndRecvSelected(AH_OUTBOX__CBOX *cbox,
 					int timeout,
 					uint32_t jqflags,
-					uint32_t jqmask){
+					uint32_t jqmask,
+					uint32_t guiid){
   AH_JOBQUEUE_LIST *jqlWanted;
   AH_JOBQUEUE_LIST *jqlRest;
   int rv;
@@ -1383,7 +1404,7 @@ int AH_Outbox__CBox_SendAndRecvSelected(AH_OUTBOX__CBOX *cbox,
   cbox->todoQueues=jqlRest;
   if (AH_JobQueue_List_GetCount(jqlWanted)) {
     /* there are matching queues, handle them */
-    rv=AH_Outbox__CBox_PerformNonDialogQueues(cbox, timeout, jqlWanted);
+    rv=AH_Outbox__CBox_PerformNonDialogQueues(cbox, timeout, jqlWanted, guiid);
     if (rv) {
       DBG_ERROR(AQHBCI_LOGDOMAIN,
 		"Error performing queue (%d)", rv);
@@ -1401,11 +1422,11 @@ int AH_Outbox__CBox_SendAndRecvSelected(AH_OUTBOX__CBOX *cbox,
 
 
 
-int AH_Outbox__CBox_SendAndRecvBox(AH_OUTBOX__CBOX *cbox, int timeout){
+int AH_Outbox__CBox_SendAndRecvBox(AH_OUTBOX__CBOX *cbox, int timeout, uint32_t guiid){
   int rv;
 
   /* dialog queues */
-  rv=AH_Outbox__CBox_SendAndRecvDialogQueues(cbox, timeout);
+  rv=AH_Outbox__CBox_SendAndRecvDialogQueues(cbox, timeout, guiid);
   if (rv) {
     DBG_INFO(AQHBCI_LOGDOMAIN, "Error performing dialog queues (%d)", rv);
     return rv;
@@ -1416,7 +1437,8 @@ int AH_Outbox__CBox_SendAndRecvBox(AH_OUTBOX__CBOX *cbox, int timeout){
 					 0,
 					 AH_JOBQUEUE_FLAGS_ISDIALOG |
 					 AH_JOBQUEUE_FLAGS_SIGN |
-					 AH_JOBQUEUE_FLAGS_CRYPT);
+					 AH_JOBQUEUE_FLAGS_CRYPT,
+					 guiid);
   if (rv) {
     DBG_INFO(AQHBCI_LOGDOMAIN, "Error performing queues (-S, -C: %d)", rv);
     return rv;
@@ -1427,7 +1449,8 @@ int AH_Outbox__CBox_SendAndRecvBox(AH_OUTBOX__CBOX *cbox, int timeout){
 					 AH_JOBQUEUE_FLAGS_CRYPT,
 					 AH_JOBQUEUE_FLAGS_ISDIALOG |
 					 AH_JOBQUEUE_FLAGS_SIGN |
-					 AH_JOBQUEUE_FLAGS_CRYPT);
+					 AH_JOBQUEUE_FLAGS_CRYPT,
+					 guiid);
   if (rv) {
     DBG_INFO(AQHBCI_LOGDOMAIN, "Error performing queues (-S, +C: %d)", rv);
     return rv;
@@ -1438,7 +1461,8 @@ int AH_Outbox__CBox_SendAndRecvBox(AH_OUTBOX__CBOX *cbox, int timeout){
 					 AH_JOBQUEUE_FLAGS_SIGN,
 					 AH_JOBQUEUE_FLAGS_ISDIALOG |
 					 AH_JOBQUEUE_FLAGS_SIGN |
-					 AH_JOBQUEUE_FLAGS_CRYPT);
+					 AH_JOBQUEUE_FLAGS_CRYPT,
+					 guiid);
   if (rv) {
     DBG_INFO(AQHBCI_LOGDOMAIN, "Error performing queues (+S, -C: %d)", rv);
     return rv;
@@ -1450,7 +1474,8 @@ int AH_Outbox__CBox_SendAndRecvBox(AH_OUTBOX__CBOX *cbox, int timeout){
 					 AH_JOBQUEUE_FLAGS_CRYPT,
 					 AH_JOBQUEUE_FLAGS_ISDIALOG |
 					 AH_JOBQUEUE_FLAGS_SIGN |
-					 AH_JOBQUEUE_FLAGS_CRYPT);
+					 AH_JOBQUEUE_FLAGS_CRYPT,
+					 guiid);
   if (rv) {
     DBG_INFO(AQHBCI_LOGDOMAIN, "Error performing queues (+S, +C: %d)", rv);
     return rv;
@@ -1488,7 +1513,6 @@ AH_OUTBOX *AH_Outbox_new(AH_HBCI *hbci, uint32_t guiid) {
   ob->hbci=hbci;
   ob->userBoxes=AH_Outbox__CBox_List_new();
   ob->finishedJobs=AH_Job_List_new();
-  ob->guiid=guiid;
   ob->usage=1;
   return ob;
 }
@@ -1518,7 +1542,7 @@ void AH_Outbox_Attach(AH_OUTBOX *ob){
 
 
 
-int AH_Outbox_Prepare(AH_OUTBOX *ob){
+int AH_Outbox_Prepare(AH_OUTBOX *ob, uint32_t guiid){
   AH_OUTBOX__CBOX *cbox;
   unsigned int errors;
 
@@ -1532,7 +1556,7 @@ int AH_Outbox_Prepare(AH_OUTBOX *ob){
     u=AH_Outbox__CBox_GetUser(cbox);
     DBG_INFO(AQHBCI_LOGDOMAIN, "Preparing queues for customer \"%s\"",
              AB_User_GetCustomerId(u));
-    if (AH_Outbox__CBox_Prepare(cbox)) {
+    if (AH_Outbox__CBox_Prepare(cbox, guiid)) {
       DBG_INFO(AQHBCI_LOGDOMAIN, "Error preparing cbox");
       errors++;
     }
@@ -1553,6 +1577,93 @@ int AH_Outbox_Prepare(AH_OUTBOX *ob){
 
 
 
+int AH_Outbox_LockUsers(AH_OUTBOX *ob, AB_USER_LIST2 *lockedUsers, uint32_t guiid){
+  AH_OUTBOX__CBOX *cbox;
+  AB_BANKING *ab;
+
+  assert(ob);
+
+  ab=AH_HBCI_GetBankingApi(ob->hbci);
+
+  cbox=AH_Outbox__CBox_List_First(ob->userBoxes);
+  while(cbox) {
+    int rv;
+    char tbuf[256];
+
+    DBG_INFO(AQHBCI_LOGDOMAIN, "Locking customer \"%s\"",
+	     AB_User_GetCustomerId(cbox->user));
+    snprintf(tbuf, sizeof(tbuf)-1,
+	     I18N("Locking user %s"),
+	     AB_User_GetUserId(cbox->user));
+    tbuf[sizeof(tbuf)-1]=0;
+    GWEN_Gui_ProgressLog(guiid,
+			 GWEN_LoggerLevel_Info,
+			 tbuf);
+    rv=AB_Banking_BeginExclUseUser(ab, cbox->user, guiid);
+    if (rv<0) {
+      DBG_INFO(AQHBCI_LOGDOMAIN,
+	       "Could not lock customer [%s] (%d)",
+	       AB_User_GetCustomerId(cbox->user), rv);
+      snprintf(tbuf, sizeof(tbuf)-1,
+	       I18N("Could not lock user %s (%d)"),
+	       AB_User_GetUserId(cbox->user), rv);
+      tbuf[sizeof(tbuf)-1]=0;
+      GWEN_Gui_ProgressLog(guiid,
+			   GWEN_LoggerLevel_Error,
+			   tbuf);
+      AH_Outbox_UnlockUsers(ob, lockedUsers, 1, guiid); /* abandon */
+      return rv;
+    }
+    cbox->isLocked=1;
+    AB_User_List2_PushBack(lockedUsers, cbox->user);
+
+    cbox=AH_Outbox__CBox_List_Next(cbox);
+  } /* while */
+
+  return 0;
+}
+
+
+
+int AH_Outbox_UnlockUsers(AH_OUTBOX *ob, AB_USER_LIST2 *lockedUsers,
+			  int abandon, uint32_t guiid){
+  int errors=0;
+  AB_BANKING *ab;
+  AB_USER_LIST2_ITERATOR *it;
+
+  assert(ob);
+
+  ab=AH_HBCI_GetBankingApi(ob->hbci);
+  it=AB_User_List2_First(lockedUsers);
+  if (it) {
+    AB_USER *u;
+
+    u=AB_User_List2Iterator_Data(it);
+    while(u) {
+      int rv;
+
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Unlocking customer \"%s\"",
+	       AB_User_GetCustomerId(u));
+      rv=AB_Banking_EndExclUseUser(ab, u, abandon, guiid);
+      if (rv<0) {
+	DBG_WARN(AQHBCI_LOGDOMAIN,
+		 "Could not unlock customer [%s] (%d)",
+		 AB_User_GetCustomerId(u), rv);
+	errors++;
+      }
+      u=AB_User_List2Iterator_Next(it);
+    }
+    AB_User_List2Iterator_free(it);
+  }
+
+  if (errors)
+    return GWEN_ERROR_GENERIC;
+
+  return 0;
+}
+
+
+
 static AH_OUTBOX__CBOX *AH_Outbox__FindCBox(const AH_OUTBOX *ob,
                                             const AB_USER *u) {
   AH_OUTBOX__CBOX *cbox;
@@ -1641,13 +1752,13 @@ void AH_Outbox_AddPendingJob(AH_OUTBOX *ob, AB_JOB *bj){
 
 
 
-int AH_Outbox_StartSending(AH_OUTBOX *ob) {
-  return AH_Outbox_Prepare(ob);
+int AH_Outbox_StartSending(AH_OUTBOX *ob, uint32_t guiid) {
+  return AH_Outbox_Prepare(ob, guiid);
 }
 
 
 
-void AH_Outbox__FinishCBox(AH_OUTBOX *ob, AH_OUTBOX__CBOX *cbox){
+void AH_Outbox__FinishCBox(AH_OUTBOX *ob, AH_OUTBOX__CBOX *cbox, uint32_t guiid){
   AH_JOB_LIST *jl;
   AH_JOB *j;
 
@@ -1664,7 +1775,7 @@ void AH_Outbox__FinishCBox(AH_OUTBOX *ob, AH_OUTBOX__CBOX *cbox){
     AH_Job_List_Del(j);
     st=AH_Job_GetStatus(j);
     if (st==AH_JobStatusAnswered) {
-      rv=AH_Job_Process(j, ob->context, cbox->guiid);
+      rv=AH_Job_Process(j, ob->context, guiid);
       if (rv) {
         char buf[256];
 
@@ -1678,10 +1789,9 @@ void AH_Outbox__FinishCBox(AH_OUTBOX *ob, AH_OUTBOX__CBOX *cbox){
                  AH_Job_GetName(j));
 	AH_Job_SetStatus(j, AH_JobStatusError);
 
-        GWEN_Gui_ProgressLog(
-                               0,
-                               GWEN_LoggerLevel_Error,
-                               buf);
+        GWEN_Gui_ProgressLog(guiid,
+			     GWEN_LoggerLevel_Error,
+			     buf);
       }
     }
     else {
@@ -1692,16 +1802,47 @@ void AH_Outbox__FinishCBox(AH_OUTBOX *ob, AH_OUTBOX__CBOX *cbox){
     AH_Job_List_Add(j, ob->finishedJobs);
   } /* while */
   AH_Job_List_free(jl);
+
+#if 0
+  /* unlock customer if necessary */
+  if (cbox->isLocked) {
+    if (cbox->isLocked) {
+      int rv;
+      char tbuf[256];
+
+      snprintf(tbuf, sizeof(tbuf)-1,
+	       I18N("Unlocking user %s"),
+	       AB_User_GetUserId(cbox->user));
+      tbuf[sizeof(tbuf)-1]=0;
+      GWEN_Gui_ProgressLog(guiid,
+			   GWEN_LoggerLevel_Info,
+                           tbuf);
+      DBG_INFO(AQHBCI_LOGDOMAIN,
+	       "Unlocking customer \"%s\"",
+	       AB_User_GetCustomerId(cbox->user));
+      rv=AB_Banking_EndExclUseUser(AH_HBCI_GetBankingApi(cbox->hbci),
+				   cbox->user,
+				   0,
+				   guiid);
+      cbox->isLocked=0;
+      if (rv<0) {
+	DBG_WARN(AQHBCI_LOGDOMAIN,
+		 "Could not unlock customer [%s] (%d)",
+		 AB_User_GetCustomerId(cbox->user), rv);
+      }
+    }
+  }
+#endif
 }
 
 
 
-static void AH_Outbox__FinishOutbox(AH_OUTBOX *ob) {
+static void AH_Outbox__FinishOutbox(AH_OUTBOX *ob, uint32_t guiid) {
   AH_OUTBOX__CBOX *cbox;
 
   assert(ob);
   while((cbox=AH_Outbox__CBox_List_First(ob->userBoxes))) {
-    AH_Outbox__FinishCBox(ob, cbox);
+    AH_Outbox__FinishCBox(ob, cbox, guiid);
     AH_Outbox__CBox_List_Del(cbox);
     AH_Outbox__CBox_free(cbox);
   } /* while */
@@ -1709,7 +1850,7 @@ static void AH_Outbox__FinishOutbox(AH_OUTBOX *ob) {
 
 
 
-int AH_Outbox_SendAndRecv(AH_OUTBOX *ob, int timeout){
+int AH_Outbox_SendAndRecv(AH_OUTBOX *ob, int timeout, uint32_t guiid){
   AH_OUTBOX__CBOX *cbox;
   int rv;
   int errors;
@@ -1723,33 +1864,34 @@ int AH_Outbox_SendAndRecv(AH_OUTBOX *ob, int timeout){
              "Sending next message for customer \"%s\"",
              AB_User_GetCustomerId(u));
 
-    rv=AH_Outbox__CBox_SendAndRecvBox(cbox, timeout);
-    AH_Outbox__FinishCBox(ob, cbox);
+    rv=AH_Outbox__CBox_SendAndRecvBox(cbox, timeout, guiid);
+    AH_Outbox__FinishCBox(ob, cbox, guiid);
     AH_Outbox__CBox_List_Del(cbox);
     AH_Outbox__CBox_free(cbox);
     if (rv)
       errors++;
     if (rv==GWEN_ERROR_USER_ABORTED) {
-      AH_Outbox__FinishOutbox(ob);
+      AH_Outbox__FinishOutbox(ob, guiid);
       return rv;
     }
   } /* while */
 
-  AH_Outbox__FinishOutbox(ob);
+  AH_Outbox__FinishOutbox(ob, guiid);
   return 0;
 }
 
 
 
-AH_JOB_LIST *AH_Outbox_GetFinishedJobs(AH_OUTBOX *ob){
+AH_JOB_LIST *AH_Outbox_GetFinishedJobs(AH_OUTBOX *ob, uint32_t guiid){
   assert(ob);
-  AH_Outbox__FinishOutbox(ob);
+  assert(ob->usage);
+  AH_Outbox__FinishOutbox(ob, guiid);
   return ob->finishedJobs;
 }
 
 
 
-void AH_Outbox_Commit(AH_OUTBOX *ob){
+void AH_Outbox_Commit(AH_OUTBOX *ob, uint32_t guiid){
   AH_JOB *j;
 
   assert(ob);
@@ -1758,7 +1900,7 @@ void AH_Outbox_Commit(AH_OUTBOX *ob){
     if (AH_Job_GetStatus(j)==AH_JobStatusAnswered) {
       /* only commit answered jobs */
       DBG_NOTICE(AQHBCI_LOGDOMAIN, "Committing job \"%s\"", AH_Job_GetName(j));
-      AH_Job_Commit(j, ob->guiid);
+      AH_Job_Commit(j, guiid);
     }
     j=AH_Job_List_Next(j);
   } /* while */
@@ -1766,7 +1908,7 @@ void AH_Outbox_Commit(AH_OUTBOX *ob){
 
 
 
-void AH_Outbox_CommitSystemData(AH_OUTBOX *ob){
+void AH_Outbox_CommitSystemData(AH_OUTBOX *ob, uint32_t guiid){
   AH_JOB *j;
 
   assert(ob);
@@ -1775,7 +1917,7 @@ void AH_Outbox_CommitSystemData(AH_OUTBOX *ob){
     if (AH_Job_GetStatus(j)==AH_JobStatusAnswered) {
       /* only commit answered jobs */
       DBG_NOTICE(AQHBCI_LOGDOMAIN, "Committing job \"%s\"", AH_Job_GetName(j));
-      AH_Job_DefaultCommitHandler(j, ob->guiid);
+      AH_Job_DefaultCommitHandler(j, guiid);
     }
     j=AH_Job_List_Next(j);
   } /* while */
@@ -1783,7 +1925,7 @@ void AH_Outbox_CommitSystemData(AH_OUTBOX *ob){
 
 
 
-void AH_Outbox_Process(AH_OUTBOX *ob){
+void AH_Outbox_Process(AH_OUTBOX *ob, uint32_t guiid){
   AH_JOB *j;
 
   assert(ob);
@@ -1795,7 +1937,7 @@ void AH_Outbox_Process(AH_OUTBOX *ob){
       /* only process answered jobs */
       DBG_DEBUG(AQHBCI_LOGDOMAIN,
 		"Processing job \"%s\"", AH_Job_GetName(j));
-      rv=AH_Job_Process(j, ob->context, ob->guiid);
+      rv=AH_Job_Process(j, ob->context, guiid);
       if (rv) {
 	char buf[256];
 
@@ -1926,7 +2068,7 @@ unsigned int AH_Outbox_CountFinishedJobs(AH_OUTBOX *ob){
 
 
 
-int AH_Outbox__Execute(AH_OUTBOX *ob){
+int AH_Outbox__Execute(AH_OUTBOX *ob, uint32_t guiid){
   unsigned int loop;
   unsigned int jobCount;
   int rv;
@@ -1942,14 +2084,14 @@ int AH_Outbox__Execute(AH_OUTBOX *ob){
 		       GWEN_LoggerLevel_Notice,
 		       I18N("AqHBCI started"));
 
-  rv=AH_Outbox_StartSending(ob);
+  rv=AH_Outbox_StartSending(ob, guiid);
   if (rv) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not start sending outbox.");
     return rv;
   }
 
   loop=0;
-  rv=AH_Outbox_SendAndRecv(ob, AH_HBCI_GetTransferTimeout(ob->hbci));
+  rv=AH_Outbox_SendAndRecv(ob, AH_HBCI_GetTransferTimeout(ob->hbci), guiid);
   if (rv) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "Error while sending outbox.");
     return rv;
@@ -1958,7 +2100,7 @@ int AH_Outbox__Execute(AH_OUTBOX *ob){
   rv=AB_Banking_ExecutionProgress(AH_HBCI_GetBankingApi(ob->hbci), 0);
   if (rv) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "User aborted");
-    AH_Outbox__FinishOutbox(ob);
+    AH_Outbox__FinishOutbox(ob, guiid);
     return rv;
   }
 
@@ -1972,11 +2114,14 @@ int AH_Outbox__Execute(AH_OUTBOX *ob){
 
 int AH_Outbox_Execute(AH_OUTBOX *ob,
                       AB_IMEXPORTER_CONTEXT *ctx,
-                      int withProgress, int nounmount) {
+		      int withProgress, int nounmount, int doLock,
+		      uint32_t guiid) {
   int rv;
   uint32_t pid=0;
+  AB_USER_LIST2 *lockedUsers = NULL;
 
   assert(ob);
+
   if (withProgress) {
     pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
 			       GWEN_GUI_PROGRESS_ALLOW_EMBED |
@@ -1986,22 +2131,65 @@ int AH_Outbox_Execute(AH_OUTBOX *ob,
 			       I18N("Now the jobs are send via their "
 				    "backends to the credit institutes."),
 			       AH_Outbox_CountTodoJobs(ob),
-			       0);
+			       guiid);
   }
+  else
+    pid=guiid;
 
   ob->context=ctx;
-  rv=AH_Outbox__Execute(ob);
+
+  if (doLock) {
+    GWEN_Gui_ProgressLog(pid,
+			 GWEN_LoggerLevel_Info,
+			 I18N("Locking users"));
+    lockedUsers=AB_User_List2_new();
+    rv=AH_Outbox_LockUsers(ob, lockedUsers, pid);
+    if (rv<0) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+      GWEN_Gui_ProgressLog(pid,
+			   GWEN_LoggerLevel_Error,
+			   I18N("Unable to lock users"));
+    }
+  }
+  else
+    rv=0;
+
+  if (rv==0) {
+    GWEN_Gui_ProgressLog(pid,
+			 GWEN_LoggerLevel_Info,
+			 I18N("Executing HBCI jobs"));
+    rv=AH_Outbox__Execute(ob, pid);
+    if (rv<0) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    }
+    if (doLock) {
+      int rv2;
+
+      rv2=AH_Outbox_UnlockUsers(ob, lockedUsers, 0, pid);
+      if (rv2<0) {
+	DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+	GWEN_Gui_ProgressLog(pid,
+			     GWEN_LoggerLevel_Error,
+			     I18N("Unable to unlock users"));
+      }
+      if (rv==0 && rv2!=0)
+	rv=rv2;
+    }
+  }
+
   /* unmount currently mounted medium */
   if (!nounmount)
     AB_Banking_ClearCryptTokenList(AH_HBCI_GetBankingApi(ob->hbci), pid);
   if (withProgress) {
     GWEN_Gui_ProgressEnd(pid);
   }
+
   ob->context=0;
   return rv;
 }
 
 
+
 AH_JOB *AH_Outbox__FindTransferJobInCheckJobList(const AH_JOB_LIST *jl,
                                                  AB_USER *u,
                                                  AB_ACCOUNT *a,
