@@ -99,7 +99,8 @@ AH_JOB *AH_Job_new(const char *name,
                                          0,
                                          name);
   if (!node) {
-    DBG_ERROR(AQHBCI_LOGDOMAIN, "Job \"%s\" not supported by local XML files", name);
+    DBG_INFO(AQHBCI_LOGDOMAIN,
+	     "Job \"%s\" not supported by local XML files", name);
     AH_Job_free(j);
     return 0;
   }
@@ -186,10 +187,20 @@ AH_JOB *AH_Job_new(const char *name,
 
     if (!jobBPD) {
       if (needsBPD) {
-	/* no BPD when needed, error */
-        DBG_ERROR(AQHBCI_LOGDOMAIN,
-                  "Job \"%s\" not supported by your bank",
-                  name);
+	if (AH_User_GetCryptMode(u)!=AH_CryptMode_Pintan &&
+	    strcasecmp(name, "JobTan")==0) {
+	  /* lower loglevel for JobTan in non-PINTAN mode because this is
+	   * often confusing */
+	  DBG_INFO(AQHBCI_LOGDOMAIN,
+		   "Job \"%s\" not supported by your bank",
+		   name);
+	}
+	else {
+	  /* no BPD when needed, error */
+	  DBG_ERROR(AQHBCI_LOGDOMAIN,
+		    "Job \"%s\" not supported by your bank",
+		    name);
+	}
 	AH_Job_free(j);
 	return 0;
       }
@@ -199,6 +210,7 @@ AH_JOB *AH_Job_new(const char *name,
       /* sample some variables from BPD jobs */
       j->minSigs=GWEN_DB_GetIntValue(jobBPD, "minsigs", 0, 0);
       j->secProfile=GWEN_DB_GetIntValue(jobBPD, "secProfile", 0, 1);
+      j->secClass=GWEN_DB_GetIntValue(jobBPD, "securityClass", 0, 0);
       j->jobsPerMsg=GWEN_DB_GetIntValue(jobBPD, "jobspermsg", 0, 0);
     }
   } /* if paramName */
@@ -487,6 +499,11 @@ int AH_Job_PrepareNextMessage(AH_JOB *j) {
     else
       j->flags&=~AH_JOB_FLAGS_NOITAN;
 
+    if (atoi(GWEN_XMLNode_GetProperty(j->msgNode, "ignerrors", "0"))!=0)
+      j->flags|=AH_JOB_FLAGS_IGNORE_ERROR;
+    else
+      j->flags&=~AH_JOB_FLAGS_IGNORE_ERROR;
+
     j->flags|=AH_JOB_FLAGS_HASMOREMSGS;
     return 1;
   }
@@ -541,6 +558,14 @@ int AH_Job_GetSecurityProfile(const AH_JOB *j){
 
 
 
+int AH_Job_GetSecurityClass(const AH_JOB *j) {
+  assert(j);
+  assert(j->usage);
+  return j->secClass;
+}
+
+
+
 int AH_Job_GetJobsPerMsg(const AH_JOB *j){
   assert(j);
   assert(j->usage);
@@ -1396,6 +1421,26 @@ int AH_Job_CommitSystemData(AH_JOB *j, uint32_t guiid) {
 	/* TODO: Read the key received and ask the user to accept it */
       }
 
+      else if (strcasecmp(GWEN_DB_GroupName(dbRd), "SecurityMethods")==0){
+        GWEN_DB_NODE *dbT;
+
+        dbT=GWEN_DB_FindFirstGroup(dbRd, "SecProfile");
+	while(dbT) {
+	  const char *code;
+          int version;
+  
+	  code=GWEN_DB_GetCharValue(dbT, "code", 0, NULL);
+	  version=GWEN_DB_GetIntValue(dbT, "version", 0, -1);
+	  if (code && (version>0)) {
+	    DBG_ERROR(AQHBCI_LOGDOMAIN,
+		      "Bank supports mode %s %d",
+		      code, version);
+	    /* TODO: store possible modes */
+	  }
+	  dbT=GWEN_DB_FindNextGroup(dbT, "SecProfile");
+	} /* while */
+      }
+
       else {
         GWEN_XMLNODE *bpdn;
         int segver;
@@ -1494,12 +1539,15 @@ int AH_Job_CommitSystemData(AH_JOB *j, uint32_t guiid) {
                    "Job contains account data");
 
         /* account data found */
-        accountId=GWEN_DB_GetCharValue(dbRd, "accountId", 0, 0);
-        assert(accountId);
+	accountId=GWEN_DB_GetCharValue(dbRd, "accountId", 0, 0);
+	if (accountId==NULL)
+	  accountId=I18N("AH_JOB|-- no account id --");
         accountName=GWEN_DB_GetCharValue(dbRd, "account/name", 0, 0);
         userName=GWEN_DB_GetCharValue(dbRd, "name1", 0, 0);
 	iban=GWEN_DB_GetCharValue(dbRd, "iban", 0, 0);
-        bankCode=GWEN_DB_GetCharValue(dbRd, "bankCode", 0, 0);
+	bankCode=GWEN_DB_GetCharValue(dbRd, "bankCode", 0, 0);
+	if (bankCode==NULL)
+	  bankCode=I18N("AH_JOB|-- no bank code --");
         assert(bankCode);
         custId=GWEN_DB_GetCharValue(dbRd, "customer", 0, 0);
         assert(custId);
@@ -2044,9 +2092,17 @@ const char *AH_Job_GetUsedTan(const AH_JOB *j){
 void AH_Job_SetUsedTan(AH_JOB *j, const char *s){
   assert(j);
   assert(j->usage);
+
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Changing TAN in job [%s](%08x) from [%s] to [%s]",
+	   j->name, j->id,
+	   (j->usedTan)?(j->usedTan):"(empty)",
+	   s?s:"(empty)");
   free(j->usedTan);
-  if (s) j->usedTan=strdup(s);
-  else j->usedTan=0;
+  if (s) {
+    j->usedTan=strdup(s);
+  }
+  else
+    j->usedTan=0;
 }
 
 
