@@ -88,6 +88,7 @@ AH_JOBQUEUE *AH_JobQueue_fromQueue(AH_JOBQUEUE *oldq){
   jq=AH_JobQueue_new(oldq->user);
   jq->signers=GWEN_StringList_dup(oldq->signers);
   jq->secProfile=oldq->secProfile;
+  jq->secClass=oldq->secClass;
   if (oldq->usedTan)
     jq->usedTan=strdup(oldq->usedTan);
   if (oldq->usedPin)
@@ -217,6 +218,17 @@ AH_JOBQUEUE_ADDRESULT AH_JobQueue_AddJob(AH_JOBQUEUE *jq, AH_JOB *j){
       return AH_JobQueueAddResultJobLimit;
     }
 
+    /* check security class */
+    if (jq->secClass==0)
+      jq->secClass=AH_Job_GetSecurityClass(j);
+    else {
+      if (jq->secClass!=AH_Job_GetSecurityClass(j)) {
+	DBG_INFO(AQHBCI_LOGDOMAIN, "Job's security class doesn't match that of the queue (%d != %d)",
+		 jq->secClass, AH_Job_GetSecurityClass(j));
+	return AH_JobQueueAddResultJobLimit;
+      }
+    }
+
     /* check for signers */
     if (!jobCount && !GWEN_StringList_Count(jq->signers)) {
       const GWEN_STRINGLIST *sl;
@@ -324,6 +336,7 @@ AH_MSG *AH_JobQueue_ToMessage(AH_JOBQUEUE *jq, AH_DIALOG *dlg){
   msg=AH_Msg_new(dlg);
   AH_Msg_SetHbciVersion(msg, AH_User_GetHbciVersion(jq->user));
   AH_Msg_SetSecurityProfile(msg, jq->secProfile);
+  AH_Msg_SetSecurityClass(msg, jq->secClass);
 
   if (AH_JobQueue_GetFlags(jq) & AH_JOBQUEUE_FLAGS_NEEDTAN) {
     DBG_NOTICE(AQHBCI_LOGDOMAIN, "Queue needs a TAN");
@@ -475,7 +488,7 @@ AH_MSG *AH_JobQueue_ToMessage(AH_JOBQUEUE *jq, AH_DIALOG *dlg){
 
 
 
-int AH_JobQueue__CheckTans(AH_JOBQUEUE *jq){
+int AH_JobQueue__CheckTans(AH_JOBQUEUE *jq, uint32_t guiid){
   AH_JOB *j;
 
   assert(jq);
@@ -492,16 +505,24 @@ int AH_JobQueue__CheckTans(AH_JOBQUEUE *jq){
       int rv;
 
       if (AH_Job_GetFlags(j) & AH_JOB_FLAGS_TANUSED) {
-        DBG_INFO(AQHBCI_LOGDOMAIN,
+	char tbuf[256];
+
+	DBG_INFO(AQHBCI_LOGDOMAIN,
 		 "TAN \"%s\" used", tan);
+	snprintf(tbuf, sizeof(tbuf)-1,
+		 I18N("TAN \"%s\" has been used, please strike it out."),
+		 tan);
+	tbuf[sizeof(tbuf)-1]=0;
+	GWEN_Gui_ProgressLog(guiid,
+			     GWEN_LoggerLevel_Notice,
+			     tbuf);
 	rv=AH_User_SetTanStatus(jq->user,
 				NULL, /* no challenge here */
 				tan,
 				GWEN_Gui_PasswordStatus_Used);
       }
       else {
-        DBG_INFO(AQHBCI_LOGDOMAIN,
-                 "TAN \"%s\" not used", tan);
+	DBG_INFO(AQHBCI_LOGDOMAIN, "TAN not used");
 	rv=AH_User_SetTanStatus(jq->user,
 				NULL, /* no challenge here */
 				tan,
@@ -533,7 +554,9 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
   int rv;
   int dialogAborted=0;
   int abortQueue=0;
+  int badPin=0;
   GWEN_STRINGLISTENTRY *se;
+  uint32_t guiid;
 
   assert(jq);
   assert(jq->usage);
@@ -542,6 +565,7 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
 
   dlg=AH_Msg_GetDialog(msg);
   assert(dlg);
+  guiid=AH_Dialog_GetGuiId(dlg);
 
   /* log all results */
   tanRecycle=0;
@@ -582,7 +606,23 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
 	  DBG_INFO(AQHBCI_LOGDOMAIN,
 		   "Result: Warning (%d: %s)", rcode, p);
           if (rcode==3910)
-            tanRecycle=1;
+	    tanRecycle=1;
+	  else if (rcode==3920) {
+	    int i;
+
+	    AH_User_ClearTanMethodList(jq->user);
+	    for (i=0; ; i++) {
+	      int j;
+
+	      j=GWEN_DB_GetIntValue(dbResult, "param", i, 0);
+	      if (j==0)
+		break;
+	      AH_User_AddTanMethod(jq->user, j);
+	    } /* for */
+	    if (i==0)
+	      /* add single step if empty list */
+	      AH_User_AddTanMethod(jq->user, 999);
+	  }
 	  level=GWEN_LoggerLevel_Warning;
         }
         else {
@@ -616,10 +656,25 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
           else
             GWEN_Buffer_AppendString(logmsg, " (S)");
         }
-	GWEN_Gui_ProgressLog(0,
+	GWEN_Gui_ProgressLog(AH_Dialog_GetGuiId(dlg),
 			     level,
 			     GWEN_Buffer_GetStart(logmsg));
 	GWEN_Buffer_free(logmsg);
+
+	/* check for bad pins here */
+	if (rcode==9340 || rcode==9942) {
+	  DBG_ERROR(AQHBCI_LOGDOMAIN,
+		    "Bad PIN flagged: %d", rcode);
+	  badPin=1;
+	  if (jq->usedPin) {
+	    GWEN_Gui_ProgressLog(AH_Dialog_GetGuiId(dlg),
+				 GWEN_LoggerLevel_Error,
+				 I18N("PIN seems to be invalid"));
+	    AH_User_SetPinStatus(jq->user, jq->usedPin,
+				 GWEN_Gui_PasswordStatus_Bad);
+	  }
+	}
+
         dbResult=GWEN_DB_FindNextGroup(dbResult, "result");
       } /* while results */
     }
@@ -772,20 +827,24 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
 	  while(dbResult) {
 	    int rcode;
 	    const char *p;
-  
+
 	    rcode=GWEN_DB_GetIntValue(dbResult, "resultcode", 0, 0);
 	    p=GWEN_DB_GetCharValue(dbResult, "text", 0, "");
 	    if (rcode>=9000 && rcode<10000) {
 	      DBG_INFO(AQHBCI_LOGDOMAIN,
 		       "Segment result: Error (%d: %s)", rcode, p);
-	      AH_Job_AddFlags(j, AH_JOB_FLAGS_HASERRORS);
-	      AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASERRORS);
+	      if (!(AH_Job_GetFlags(j) & AH_JOB_FLAGS_IGNORE_ERROR)) {
+		AH_Job_AddFlags(j, AH_JOB_FLAGS_HASERRORS);
+		AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASERRORS);
+	      }
 	    }
 	    else if (rcode>=3000 && rcode<4000) {
 	      DBG_INFO(AQHBCI_LOGDOMAIN,
 		       "Segment result: Warning (%d: %s)", rcode, p);
-	      AH_Job_AddFlags(j, AH_JOB_FLAGS_HASWARNINGS);
-	      AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASWARNINGS);
+	      if (!(AH_Job_GetFlags(j) & AH_JOB_FLAGS_IGNORE_ERROR)) {
+		AH_Job_AddFlags(j, AH_JOB_FLAGS_HASWARNINGS);
+		AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASWARNINGS);
+	      }
 	    }
 	    else {
 	      DBG_INFO(AQHBCI_LOGDOMAIN,
@@ -811,26 +870,33 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
         /* add response to all jobs (as queue response) and to queue */
         plusFlags=0;
         if (strcasecmp(GWEN_DB_GroupName(dbCurr), "MsgResult")==0) {
-          int rcode;
-          const char *p;
-          /* FIXME: This code will never be used, I guess, since
-           * a MsgResult wil most likely not have a reference segment... */
-          rcode=GWEN_DB_GetIntValue(dbCurr, "result/resultcode", 0, 0);
-          p=GWEN_DB_GetCharValue(dbCurr, "result/text", 0, "");
-          if (rcode>=9000 && rcode<10000) {
-            DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Error (%d: %s)", rcode, p);
-            plusFlags|=AH_JOB_FLAGS_HASERRORS;
-            AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASERRORS);
-          }
-          else if (rcode>=3000 && rcode<4000) {
-            DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Warning (%d: %s)", rcode, p);
-            plusFlags|=AH_JOB_FLAGS_HASWARNINGS;
-            AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASWARNINGS);
-          }
-          else {
-            DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Ok (%d: %s)", rcode, p);
-          }
-        }
+          GWEN_DB_NODE *dbResult;
+
+          dbResult=GWEN_DB_FindFirstGroup(dbCurr, "result");
+          while(dbResult) {
+	    int rcode;
+	    const char *p;
+
+	    /* FIXME: This code will never be used, I guess, since
+	     * a MsgResult wil most likely not have a reference segment... */
+	    rcode=GWEN_DB_GetIntValue(dbResult, "resultcode", 0, 0);
+	    p=GWEN_DB_GetCharValue(dbResult, "text", 0, "");
+	    if (rcode>=9000 && rcode<10000) {
+	      DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Error (%d: %s)", rcode, p);
+	      plusFlags|=AH_JOB_FLAGS_HASERRORS;
+	      AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASERRORS);
+	    }
+	    else if (rcode>=3000 && rcode<4000) {
+	      DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Warning (%d: %s)", rcode, p);
+	      plusFlags|=AH_JOB_FLAGS_HASWARNINGS;
+	      AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASWARNINGS);
+	    }
+	    else {
+	      DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Ok (%d: %s)", rcode, p);
+	    }
+	    dbResult=GWEN_DB_FindNextGroup(dbResult, "result");
+	  }
+	}
         else if (strcasecmp(GWEN_DB_GroupName(dbCurr), "SegResult")==0) {
           GWEN_DB_NODE *dbResult;
 
@@ -838,18 +904,11 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
           while(dbResult) {
             int rcode;
 
-            rcode=GWEN_DB_GetIntValue(dbResult, "resultcode", 0, 0);
-            if (rcode==9340) {
-              DBG_ERROR(AQHBCI_LOGDOMAIN,
-                        "Found a segresult: %d", rcode);
-	      if (jq->usedPin) {
-		AH_User_SetPinStatus(jq->user, jq->usedPin,
-				     GWEN_Gui_PasswordStatus_Bad);
-	      }
-            }
-            dbResult=GWEN_DB_FindNextGroup(dbResult, "result");
-          } /* while */
-        } /* if segresult */
+	    rcode=GWEN_DB_GetIntValue(dbResult, "resultcode", 0, 0);
+	    /* nothing to do right now */
+	    dbResult=GWEN_DB_FindNextGroup(dbResult, "result");
+	  } /* while */
+	} /* if segresult */
 
         j=AH_Job_List_First(jq->jobs);
         while(j) {
@@ -858,8 +917,10 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
           st=AH_Job_GetStatus(j);
           if (st==AH_JobStatusSent ||
               st==AH_JobStatusAnswered) {
-	    AH_Job_AddFlags(j, plusFlags);
-            AH_Job_AddResponse(j, GWEN_DB_Group_dup(dbResponse));
+	    if (!(AH_Job_GetFlags(j) & AH_JOB_FLAGS_IGNORE_ERROR)) {
+	      AH_Job_AddFlags(j, plusFlags);
+	      AH_Job_AddResponse(j, GWEN_DB_Group_dup(dbResponse));
+	    }
             AH_Job_SetStatus(j, AH_JobStatusAnswered);
           }
           else {
@@ -884,24 +945,30 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
       /* add response to all jobs (as queue response) and to queue */
       plusFlags=0;
       if (strcasecmp(GWEN_DB_GroupName(dbCurr), "MsgResult")==0) {
-        int rcode;
-        const char *p;
+        GWEN_DB_NODE *dbResult;
 
-        rcode=GWEN_DB_GetIntValue(dbCurr, "result/resultcode", 0, 0);
-        p=GWEN_DB_GetCharValue(dbCurr, "result/text", 0, "");
-        if (rcode>=9000 && rcode<10000) {
-          DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Error (%d: %s)", rcode, p);
-          plusFlags|=AH_JOB_FLAGS_HASERRORS;
-          AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASERRORS);
-        }
-        else if (rcode>=3000 && rcode<4000) {
-          DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Warning (%d: %s)", rcode, p);
-          plusFlags|=AH_JOB_FLAGS_HASWARNINGS;
-          AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASWARNINGS);
-        }
-        else {
-          DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Ok (%d: %s)", rcode, p);
-        }
+	dbResult=GWEN_DB_FindFirstGroup(dbCurr, "result");
+	while(dbResult) {
+	  int rcode;
+	  const char *p;
+
+	  rcode=GWEN_DB_GetIntValue(dbResult, "resultcode", 0, 0);
+	  p=GWEN_DB_GetCharValue(dbResult, "text", 0, "");
+	  if (rcode>=9000 && rcode<10000) {
+	    DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Error (%d: %s)", rcode, p);
+	    plusFlags|=AH_JOB_FLAGS_HASERRORS;
+	    AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASERRORS);
+	  }
+	  else if (rcode>=3000 && rcode<4000) {
+	    DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Warning (%d: %s)", rcode, p);
+	    plusFlags|=AH_JOB_FLAGS_HASWARNINGS;
+	    AH_JobQueue_AddFlags(jq, AH_JOBQUEUE_FLAGS_HASWARNINGS);
+	  }
+	  else {
+	    DBG_INFO(AQHBCI_LOGDOMAIN, "Msg result: Ok (%d: %s)", rcode, p);
+	  }
+          dbResult=GWEN_DB_FindNextGroup(dbResult, "Result");
+	}
       }
 
       j=AH_Job_List_First(jq->jobs);
@@ -911,10 +978,12 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
         st=AH_Job_GetStatus(j);
         if (st==AH_JobStatusSent ||
             st==AH_JobStatusAnswered) {
-          AH_Job_AddFlags(j, plusFlags);
-          AH_Job_AddResponse(j, GWEN_DB_Group_dup(dbResponse));
-          AH_Job_SetStatus(j, AH_JobStatusAnswered);
-        }
+	  if (!(AH_Job_GetFlags(j) & AH_JOB_FLAGS_IGNORE_ERROR)) {
+	    AH_Job_AddFlags(j, plusFlags);
+	    AH_Job_AddResponse(j, GWEN_DB_Group_dup(dbResponse));
+	  }
+	  AH_Job_SetStatus(j, AH_JobStatusAnswered);
+	}
         j=AH_Job_List_Next(j);
       } /* while */
       GWEN_DB_Group_free(dbResponse);
@@ -944,13 +1013,14 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
       }
     }
     else {
-      DBG_WARN(AQHBCI_LOGDOMAIN, "No TAN");
+      DBG_WARN(AQHBCI_LOGDOMAIN, "No TAN in job [%s]",
+	       AH_Job_GetName(j));
     }
     j=AH_Job_List_Next(j);
   } /* while */
 
   /* tell the application/medium about used and unused TANs */
-  rv=AH_JobQueue__CheckTans(jq);
+  rv=AH_JobQueue__CheckTans(jq, guiid);
   if (rv) {
     DBG_INFO(AQHBCI_LOGDOMAIN, "Error checking TANs (%d)", rv);
   }
@@ -965,6 +1035,18 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
     return GWEN_ERROR_ABORTED;
   }
 
+  if (!badPin) {
+    DBG_INFO(AQHBCI_LOGDOMAIN,
+	     "Dialog not aborted, assuming correct PIN");
+    if (jq->usedPin) {
+      GWEN_Gui_ProgressLog(AH_Dialog_GetGuiId(dlg),
+			   GWEN_LoggerLevel_Info,
+			   I18N("Dialog not aborted, assuming PIN is ok"));
+      AH_User_SetPinStatus(jq->user, jq->usedPin,
+			   GWEN_Gui_PasswordStatus_Ok);
+    }
+  }
+
   return rv;
 }
 
