@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: hbci_p.h 950 2006-02-22 19:39:02Z aquamaniac $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -16,7 +16,10 @@
 
 #include "hbci_l.h"
 
-#define AH_REGKEY_PATHS       "Software\\AqHbci\\Paths"
+/* Note: We use the key "AqBanking" because from the windows registry
+ * point of view, these plugins all belong to the large AqBanking
+ * package. */
+#define AH_REGKEY_PATHS       "Software\\AqBanking\\Paths"
 #define AH_REGKEY_XMLDATADIR  "xmldatadir"
 
 #define AH_PM_LIBNAME       "aqhbci"
@@ -36,18 +39,14 @@ struct AH_HBCI {
 
   GWEN_XMLNODE *defs;
 
-  AH_MEDIUM_LIST *activeMedia;
-
-  GWEN_TYPE_UINT32 counter;
-  AH_MEDIUM *currentMedium;
-  GWEN_TYPE_UINT32 lastMediumId;
+  uint32_t counter;
 
   GWEN_DB_NODE *sharedRuntimeData;
 
   int transferTimeout;
   int connectTimeout;
 
-  GWEN_TYPE_UINT32 lastVersion;
+  uint32_t lastVersion;
 
 };
 
@@ -63,12 +62,6 @@ static GWEN_DB_NODE *AH_HBCI_LoadSettings(const char *path);
 static int AH_HBCI_AddDefinitions(AH_HBCI *hbci, GWEN_XMLNODE *node);
 static GWEN_XMLNODE *AH_HBCI_LoadDefaultXmlFiles(const AH_HBCI *hbci);
 
-static int AH_HBCI__LoadMedia(AH_HBCI *hbci, GWEN_DB_NODE *db);
-static int AH_HBCI__SaveMedia(AH_HBCI *hbci, GWEN_DB_NODE *db);
-
-static int AH_HBCI__AddMedium(AH_HBCI *hbci, AH_MEDIUM *m);
-
-
 #endif /* GWHBCI_HBCI_P_H */
 
 
