@@ -204,6 +204,20 @@ void AH_Msg_SetSecurityProfile(AH_MSG *hmsg, int i) {
 
 
 
+int AH_Msg_GetSecurityClass(const AH_MSG *hmsg) {
+  assert(hmsg);
+  return hmsg->secClass;
+}
+
+
+
+void AH_Msg_SetSecurityClass(AH_MSG *hmsg, int i) {
+  assert(hmsg);
+  hmsg->secClass=i;
+}
+
+
+
 /* --------------------------------------------------------------- FUNCTION */
 int AH_Msg_AddMsgTail(AH_MSG *hmsg){
   GWEN_XMLNODE *node;
@@ -1642,7 +1656,8 @@ void AH_Msg_LogMessage(AH_MSG *msg,
   
 	if (strncasecmp(p, "HNSHA:", 6)==0)
 	  rv=AH_Msg__AnonHnsha(p, slen, io);
-	else if (strncasecmp(p, "HKPAE:", 6)==0)
+	else if (strncasecmp(p, "HKPAE:", 6)==0 ||
+		 strncasecmp(p, "DKPAE:", 6)==0)
 	  rv=AH_Msg__AnonHkpae(p, slen, io);
 	/* add more segments with confidential data here */
 	else {
@@ -1845,6 +1860,11 @@ const char *AH_Msg_GetCrypterId(const AH_MSG *hmsg) {
 
 
 #include "msgcrypt_ddv.c"
+#include "msgcrypt_rdh1.c"
+#include "msgcrypt_rdh2.c"
+#include "msgcrypt_rdh3.c"
+#include "msgcrypt_rdh5.c"
+#include "msgcrypt_rdh10.c"
 #include "msgcrypt_rdh.c"
 #include "msgcrypt_pintan.c"
 #include "msgcrypt.inc"
