@@ -83,6 +83,9 @@ void AH_User_Flags_toDb(GWEN_DB_NODE *db, const char *name,
   if (flags & AH_USER_FLAGS_NO_BASE64)
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT, name,
 			 "noBase64");
+  if (flags & AH_USER_FLAGS_KEEP_MULTIPLE_BLANKS)
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT, name,
+			 "keepMultipleBlanks");
 }
 
 
@@ -109,6 +112,8 @@ uint32_t AH_User_Flags_fromDb(GWEN_DB_NODE *db, const char *name) {
       fl|=AH_USER_FLAGS_FORCE_SSL3;
     else if (strcasecmp(s, "noBase64")==0)
       fl|=AH_USER_FLAGS_NO_BASE64;
+    else if (strcasecmp(s, "keepMultipleBlanks")==0)
+      fl|=AH_USER_FLAGS_KEEP_MULTIPLE_BLANKS;
     else {
       DBG_WARN(AQHBCI_LOGDOMAIN, "Unknown user flag \"%s\"", s);
     }
@@ -124,6 +129,7 @@ uint32_t AH_User_Flags_fromDb(GWEN_DB_NODE *db, const char *name) {
 void AH_User_Extend(AB_USER *u, AB_PROVIDER *pro,
 		    AB_PROVIDER_EXTEND_MODE em,
 		    GWEN_DB_NODE *db) {
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Extending user with mode %d", em);
   if (em==AB_ProviderExtendMode_Create ||
       em==AB_ProviderExtendMode_Extend) {
     AH_USER *ue;
@@ -167,6 +173,11 @@ void AH_User_Extend(AB_USER *u, AB_PROVIDER *pro,
       AH_User_LoadTanMethods(u);
     }
   }
+  else if (em==AB_ProviderExtendMode_Reload) {
+    /* just reload user */
+    AH_User_ReadDb(u, db);
+    AH_User_LoadTanMethods(u);
+  }
   else {
     AH_USER *ue;
 
@@ -306,6 +317,10 @@ void AH_User_ReadDb(AB_USER *u, GWEN_DB_NODE *db) {
   ue->rdhType=GWEN_DB_GetIntValue(db, "rdhType", 0, -1);
   
   /* read supported TAN methods */
+  for (i=0; i<AH_USER_MAX_TANMETHODS; i++)
+    ue->tanMethodList[i]=-1;
+  ue->tanMethodCount=0;
+
   for (i=0; i<AH_USER_MAX_TANMETHODS; i++) {
     int method;
   
@@ -423,7 +438,7 @@ void AH_User_toDb(AB_USER *u, GWEN_DB_NODE *db) {
 			ue->tanMethodList[i]);
   }
   
-  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT,
+  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
 		      "selectedTanMethod",
 		      ue->selectedTanMethod);
 }
@@ -1068,7 +1083,8 @@ int AH_User_MkTanName(const AB_USER *u,
 int AH_User_InputPin(AB_USER *u,
 		     char *pwbuffer,
 		     int minLen, int maxLen,
-		     int flags){
+                     int flags,
+                     uint32_t guiid){
   GWEN_BUFFER *nbuf;
   int rv;
   const char *numeric_warning = "";
@@ -1122,7 +1138,7 @@ int AH_User_InputPin(AB_USER *u,
 		  "user %s at %s\n"
 		  "%s"
 		  "<html>"
-		  "Please enter the PIN for user <i>%s</i> at"
+		  "Please enter the PIN for user <i>%s</i> at "
 		  "<i>%s</i>.<br>"
 		  "%s"
 		  "</html>"),
@@ -1145,7 +1161,7 @@ int AH_User_InputPin(AB_USER *u,
 			  pwbuffer,
 			  minLen,
 			  maxLen,
-			  0);
+                          guiid);
   GWEN_Buffer_free(nbuf);
 
   return rv;
@@ -1156,7 +1172,8 @@ int AH_User_InputPin(AB_USER *u,
 int AH_User_InputPasswd(AB_USER *u,
 			char *pwbuffer,
 			int minLen, int maxLen,
-			int flags){
+                        int flags,
+                        uint32_t guiid){
   GWEN_BUFFER *nbuf;
   int rv;
   const char *numeric_warning = "";
@@ -1233,7 +1250,7 @@ int AH_User_InputPasswd(AB_USER *u,
 			  pwbuffer,
 			  minLen,
 			  maxLen,
-			  0);
+                          guiid);
   GWEN_Buffer_free(nbuf);
 
   return rv;
@@ -1244,7 +1261,8 @@ int AH_User_InputPasswd(AB_USER *u,
 int AH_User_InputTan(AB_USER *u,
 		     char *pwbuffer,
 		     int minLen,
-		     int maxLen){
+                     int maxLen,
+                     uint32_t guiid){
   int rv;
   char buffer[512];
   const char *un;
@@ -1281,7 +1299,7 @@ int AH_User_InputTan(AB_USER *u,
   AH_User_MkTanName(u, NULL, nbuf);
 
   rv=GWEN_Gui_GetPassword(GWEN_GUI_INPUT_FLAGS_TAN |
-			  GWEN_GUI_INPUT_FLAGS_NUMERIC |
+			  /*GWEN_GUI_INPUT_FLAGS_NUMERIC |*/
 			  GWEN_GUI_INPUT_FLAGS_SHOW,
 			  GWEN_Buffer_GetStart(nbuf),
 			  I18N("Enter TAN"),
@@ -1289,7 +1307,7 @@ int AH_User_InputTan(AB_USER *u,
 			  pwbuffer,
 			  minLen,
 			  maxLen,
-			  0);
+                          guiid);
   GWEN_Buffer_free(nbuf);
   AB_BankInfo_free(bi);
   return rv;
@@ -1301,7 +1319,8 @@ int AH_User_InputTanWithChallenge(AB_USER *u,
 				  const char *challenge,
 				  char *pwbuffer,
 				  int minLen,
-				  int maxLen){
+                                  int maxLen,
+                                  uint32_t guiid){
   int rv;
   char buffer[512];
   const char *un;
@@ -1346,7 +1365,7 @@ int AH_User_InputTanWithChallenge(AB_USER *u,
   nbuf=GWEN_Buffer_new(0, 256 ,0 ,1);
   AH_User_MkTanName(u, challenge, nbuf);
   rv=GWEN_Gui_GetPassword(GWEN_GUI_INPUT_FLAGS_TAN |
-			  GWEN_GUI_INPUT_FLAGS_NUMERIC |
+			  /*GWEN_GUI_INPUT_FLAGS_NUMERIC |*/
 			  GWEN_GUI_INPUT_FLAGS_SHOW,
 			  GWEN_Buffer_GetStart(nbuf),
 			  I18N("Enter TAN"),
@@ -1354,7 +1373,7 @@ int AH_User_InputTanWithChallenge(AB_USER *u,
 			  pwbuffer,
 			  minLen,
 			  maxLen,
-			  0);
+                          guiid);
   GWEN_Buffer_free(nbuf);
   AB_BankInfo_free(bi);
   return rv;
@@ -1452,6 +1471,7 @@ int AH_User_HasTanMethod(const AB_USER *u, int method) {
       return 1;
   }
 
+  DBG_INFO(AQHBCI_LOGDOMAIN, "TAN method %d not available", method);
   return 0;
 }
 
@@ -1549,6 +1569,8 @@ void AH_User_LoadTanMethods(AB_USER *u) {
   ue=GWEN_INHERIT_GETDATA(AB_USER, AH_USER, u);
   assert(ue);
 
+  AH_TanMethod_List_Clear(ue->tanMethodDescriptions);
+
   jTan=AH_Job_Tan_new(u, 1);
   if (!jTan) {
     DBG_WARN(AQHBCI_LOGDOMAIN, "Job HKTAN not available");
@@ -1561,12 +1583,19 @@ void AH_User_LoadTanMethods(AB_USER *u) {
     assert(db);
 
     dbT=GWEN_DB_FindFirstGroup(db, "tanMethod");
+    if (!dbT) {
+      DBG_ERROR(AQHBCI_LOGDOMAIN, "No tanmethod found");
+    }
     while(dbT) {
       AH_TAN_METHOD *tm;
 
       tm=AH_TanMethod_fromDb(dbT);
-      if (tm)
+      if (tm) {
+	DBG_INFO(AQHBCI_LOGDOMAIN,
+		 "Adding TAN method [%s]",
+		 AH_TanMethod_GetMethodId(tm));
 	AH_TanMethod_List_Add(tm, ue->tanMethodDescriptions);
+      }
       else {
 	DBG_WARN(AQHBCI_LOGDOMAIN, "Invalid TAN method");
 	GWEN_DB_Dump(dbT, stderr, 2);
