@@ -50,15 +50,25 @@ int addUser(AB_BANKING *ab,
             int argc,
             char **argv);
 
+int delUser(AB_BANKING *ab,
+            GWEN_DB_NODE *dbArgs,
+            int argc,
+            char **argv);
+
 int getAccounts(AB_BANKING *ab,
                 GWEN_DB_NODE *dbArgs,
                 int argc,
                 char **argv);
 
 int addAccount(AB_BANKING *ab,
-	       GWEN_DB_NODE *dbArgs,
-	       int argc,
-	       char **argv);
+             GWEN_DB_NODE *dbArgs,
+             int argc,
+             char **argv);
+
+int delAccount(AB_BANKING *ab,
+             GWEN_DB_NODE *dbArgs,
+             int argc,
+             char **argv);
 
 int getSysId(AB_BANKING *ab,
              GWEN_DB_NODE *dbArgs,
@@ -100,6 +110,16 @@ int getItanModes(AB_BANKING *ab,
                  int argc,
                  char **argv);
 
+int listItanModes(AB_BANKING *ab,
+		  GWEN_DB_NODE *dbArgs,
+		  int argc,
+		  char **argv);
+
+int setItanMode(AB_BANKING *ab,
+		GWEN_DB_NODE *dbArgs,
+		int argc,
+		char **argv);
+
 int changePin(AB_BANKING *ab,
               GWEN_DB_NODE *dbArgs,
               int argc,
@@ -125,6 +145,12 @@ int subUserFlags(AB_BANKING *ab,
 		 int argc,
 		 char **argv);
 
+int addsubAccountFlags(AB_BANKING *ab,
+		 GWEN_DB_NODE *dbArgs,
+		 int argc,
+		 char **argv,
+     int is_add );
+
 
 
 int test1(AB_BANKING *ab,
