@@ -23,7 +23,7 @@
 #include <gwenhywfar/logger.h>
 #include <gwenhywfar/debug.h>
 #include <aqbanking/banking.h>
-
+#include <aqbanking/abgui.h>
 
 #include "globals.h"
 
@@ -38,6 +38,28 @@ int EBC_Plugins_Init();
 
 
 
+static void showVersions() {
+  int vmajor, vminor, vpatchLevel, vbuild;
+
+  fprintf(stdout, "Versions:\n");
+  GWEN_Version(&vmajor,
+	       &vminor,
+	       &vpatchLevel,
+	       &vbuild);
+  fprintf(stdout, " Gwenhywfar   : %d.%d.%d.%d\n",
+	  vmajor, vminor, vpatchLevel, vbuild);
+
+  AB_Banking_GetVersion(&vmajor,
+			&vminor,
+			&vpatchLevel,
+			&vbuild);
+  fprintf(stdout, " AqBanking    : %d.%d.%d.%d\n",
+	  vmajor, vminor, vpatchLevel, vbuild);
+
+}
+
+
+
 int main(int argc, char **argv) {
   GWEN_DB_NODE *db;
   const char *s;
@@ -155,6 +177,9 @@ int main(int argc, char **argv) {
                              I18N("  adduser:\n"
                                   "    Adds a user "
                                   "(-> setup HBCI for a bank)\n\n"));
+    GWEN_Buffer_AppendString(ubuf,
+                             I18N("  deluser:\n"
+                                  "    Deletes a user.\n\n"));
     GWEN_Buffer_AppendString(ubuf,
                              I18N("  getkeys:\n"
                                   "    Requests the server's key\n\n"));
@@ -195,6 +220,9 @@ int main(int argc, char **argv) {
     GWEN_Buffer_AppendString(ubuf,
                              I18N("  addaccount:\n"
                                   "    Manually add account \n\n"));
+    GWEN_Buffer_AppendString(ubuf,
+                             I18N("  delaccount:\n"
+                                  "    Deletes account \n\n"));
 
     fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
     GWEN_Buffer_free(ubuf);
@@ -242,7 +270,10 @@ int main(int argc, char **argv) {
   s=GWEN_DB_GetCharValue(db, "charset", 0, "ISO-8859-15");
   GWEN_Gui_CGui_SetCharSet(gui, s);
   nonInteractive=GWEN_DB_GetIntValue(db, "nonInteractive", 0, 0);
-  GWEN_Gui_CGui_SetIsNonInteractive(gui, nonInteractive);
+  if (nonInteractive)
+    GWEN_Gui_AddFlags(gui, GWEN_GUI_FLAGS_NONINTERACTIVE);
+  else
+    GWEN_Gui_SubFlags(gui, GWEN_GUI_FLAGS_NONINTERACTIVE);
   pinFile=GWEN_DB_GetCharValue(db, "pinFile", 0, NULL);
   if (pinFile) {
     GWEN_DB_NODE *dbPins;
@@ -261,6 +292,7 @@ int main(int argc, char **argv) {
 
   ab=AB_Banking_new("aqhbci-tool", GWEN_DB_GetCharValue(db, "cfgfile", 0, 0),
 		    0);
+  AB_Gui_Extend(gui, ab);
 
   if (strcasecmp(cmd, "mkpinlist")==0) {
     rv=mkPinList(ab, db, argc, argv);
@@ -268,12 +300,18 @@ int main(int argc, char **argv) {
   else if (strcasecmp(cmd, "adduser")==0) {
     rv=addUser(ab, db, argc, argv);
   }
+  else if (strcasecmp(cmd, "deluser")==0) {
+    rv=delUser(ab, db, argc, argv);
+  }
   else if (strcasecmp(cmd, "getaccounts")==0) {
     rv=getAccounts(ab, db, argc, argv);
   }
   else if (strcasecmp(cmd, "addaccount")==0) {
     rv=addAccount(ab, db, argc, argv);
   }
+  else if (strcasecmp(cmd, "delaccount")==0) {
+    rv=delAccount(ab, db, argc, argv);
+  }
   else if (strcasecmp(cmd, "getsysid")==0) {
     rv=getSysId(ab, db, argc, argv);
   }
@@ -301,6 +339,12 @@ int main(int argc, char **argv) {
   else if (strcasecmp(cmd, "getitanmodes")==0) {
     rv=getItanModes(ab, db, argc, argv);
   }
+  else if (strcasecmp(cmd, "listitanmodes")==0) {
+    rv=listItanModes(ab, db, argc, argv);
+  }
+  else if (strcasecmp(cmd, "setitanmode")==0) {
+    rv=setItanMode(ab, db, argc, argv);
+  }
   else if (strcasecmp(cmd, "changepin")==0) {
     rv=changePin(ab, db, argc, argv);
   }
@@ -313,6 +357,16 @@ int main(int argc, char **argv) {
   else if (strcasecmp(cmd, "subuserflags")==0) {
     rv=subUserFlags(ab, db, argc, argv);
   }
+  else if (strcasecmp(cmd, "addaccountflags")==0) {
+    rv=addsubAccountFlags(ab, db, argc, argv, 1);
+  }
+  else if (strcasecmp(cmd, "subaccountflags")==0) {
+    rv=addsubAccountFlags(ab, db, argc, argv, 0);
+  }
+  else if (strcasecmp(cmd, "versions")==0) {
+    showVersions();
+    rv=0;
+  }
   else if (strcasecmp(cmd, "test1")==0) {
     rv=test1(ab, db, argc, argv);
   }
