@@ -118,7 +118,9 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
   const char *p;
   int isStructured;
   int code;
+  int keepMultipleBlanks;
 
+  keepMultipleBlanks=GWEN_DB_GetIntValue(cfg, "keepMultipleBlanks", 0, 1);
   p=AHB_SWIFT_Tag_GetData(tg);
   assert(p);
   isStructured=0;
@@ -143,6 +145,8 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
     GWEN_DB_SetIntValue(data, flags, "textkey", code);
 
     while(*p) {
+      int intVal;
+
       if (strlen(p)<3) {
 	DBG_ERROR(AQBANKING_LOGDOMAIN, "Bad field in :86: tag (%s)", p);
 	GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
@@ -180,7 +184,7 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
       s=(char*)GWEN_Memory_malloc(p2-p+1);
       memmove(s, p, p2-p+1);
       s[p2-p]=0;
-      AHB_SWIFT_Condense(s);
+      AHB_SWIFT_Condense(s, keepMultipleBlanks);
       DBG_DEBUG(AQBANKING_LOGDOMAIN, "Current field is %02d (%s)", id, s);
       /* now id is the field id, s points to the field content */
       if (*s) {
@@ -222,7 +226,14 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
           AHB_SWIFT__SetCharValue(data, flags, "remoteName", s);
           break;
     
-        case 34: /* Textschluesselergaenzung */
+	case 34: /* Textschluesselergaenzung */
+	  if (1==sscanf(s, "%d", &intVal)) {
+	    GWEN_DB_SetIntValue(data, flags, "textkeyExt", intVal);
+	  }
+	  else {
+	    DBG_WARN(AQBANKING_LOGDOMAIN,
+                     "Value [%s] is not a number (textkeyext)", s);
+	  }
 	  break;
 
 	case 38: /* IBAN */
@@ -240,8 +251,55 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
     } /* while */
   } /* if structured */
   else {
-    /* unstructured :86:, simply store as purpose line */
-    AHB_SWIFT__SetCharValue(data, flags, "purpose", p);
+    char *pcopy=strdup(p);
+    char *p1;
+
+    /* unstructured :86:, simply store as mutliple purpose lines */
+    p1=pcopy;
+    while(p1 && *p1) {
+      char *p2;
+
+      p2=strchr(p1, 10);
+      if (p2) {
+	*p2=0;
+	p2++;
+      }
+      if (-1!=GWEN_Text_ComparePattern(p1, "*KTO/BLZ */*", 0)) {
+	char *p3;
+	char *kto;
+
+	p3=p1;
+	while(*p3) {
+	  *p3=toupper(*p3);
+          p3++;
+	}
+	kto=strstr(p1, "KTO/BLZ ");
+	if (kto) {
+	  char *blz;
+
+          kto+=8;
+	  blz=strchr(kto, '/');
+	  if (blz) {
+	    *blz=0;
+            blz++;
+	  }
+	  p3=blz;
+	  while(*p3 && isdigit(*p3))
+	    p3++;
+	  *p3=0;
+
+	  AHB_SWIFT__SetCharValue(data, flags, "remoteBankCode", blz);
+	  AHB_SWIFT__SetCharValue(data, flags, "remoteAccountNumber", kto);
+	}
+	else {
+	  AHB_SWIFT__SetCharValue(data, flags, "purpose", p1);
+	}
+      }
+      else
+	AHB_SWIFT__SetCharValue(data, flags, "purpose", p1);
+      p1=p2;
+    }
+    free(pcopy);
   }
 
   return 0;
@@ -424,14 +482,17 @@ int AHB_SWIFT940_Parse_61(const AHB_SWIFT_TAG *tg,
   bleft-=3;
 
   /* customer reference (M) */
-  if (bleft>1) {
-    if (*p=='/' && p[1]!='/') {
-      p++;
-      bleft--;
+  if (bleft>0) {
+    if (bleft>1) {
+      if (*p=='/' && p[1]!='/') {
+	p++;
+	bleft--;
+      }
     }
 
     p2=p;
-    while(*p2 && *p2!='/' && *p2!=10) p2++;
+    while(*p2 && *p2!='/' && *p2!=10)
+      p2++;
 
     if (p2==p) {
       DBG_WARN(AQBANKING_LOGDOMAIN, "Missing customer reference (%s)", p);
@@ -793,6 +854,16 @@ int AHB_SWIFT940_Import(AHB_SWIFT_TAG_LIST *tl,
   GWEN_DB_NODE *dbTransaction=NULL;
   GWEN_DB_NODE *dbDate=NULL;
   uint32_t progressId;
+  const char *acceptTag20="*";
+  const char *rejectTag20=NULL;
+  int ignoreCurrentReport=0;
+
+  acceptTag20=GWEN_DB_GetCharValue(cfg, "acceptTag20", 0, NULL);
+  if (acceptTag20 && *acceptTag20==0)
+    acceptTag20=NULL;
+  rejectTag20=GWEN_DB_GetCharValue(cfg, "rejectTag20", 0, NULL);
+  if (rejectTag20 && *rejectTag20==0)
+    rejectTag20=NULL;
 
   dbTemplate=GWEN_DB_Group_new("template");
 
@@ -812,116 +883,149 @@ int AHB_SWIFT940_Import(AHB_SWIFT_TAG_LIST *tl,
     id=AHB_SWIFT_Tag_GetId(tg);
     assert(id);
 
-    if (strcasecmp(id, "25")==0) { /* LocalAccount */
-      if (AHB_SWIFT940_Parse_25(tg, flags, dbTemplate, cfg)) {
-        DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
-        GWEN_DB_Group_free(dbTemplate);
-	GWEN_Gui_ProgressEnd(progressId);
-        return -1;
+    if (strcasecmp(id, "20")==0) {
+      if (acceptTag20 || rejectTag20) {
+	const char *p;
+
+	p=AHB_SWIFT_Tag_GetData(tg);
+	assert(p);
+	if (rejectTag20) {
+	  if (-1!=GWEN_Text_ComparePattern(p, rejectTag20, 0)) {
+	    DBG_INFO(AQBANKING_LOGDOMAIN, "Ignoring report [%s]", p);
+	    ignoreCurrentReport=1;
+	  }
+	  else {
+	    ignoreCurrentReport=0;
+	  }
+	}
+	else if (acceptTag20) {
+	  if (-1==GWEN_Text_ComparePattern(p, acceptTag20, 0)) {
+	    DBG_INFO(AQBANKING_LOGDOMAIN,
+		     "Ignoring report [%s] (not matching [%s])",
+		     p, acceptTag20);
+	    ignoreCurrentReport=1;
+	  }
+	  else {
+	    ignoreCurrentReport=0;
+	  }
+	}
+
       }
     }
-    else if (strcasecmp(id, "60F")==0) { /* StartSaldo */
-      GWEN_DB_NODE *dbSaldo;
-      const char *curr;
-
-      /* start a new day */
-      dbDay=GWEN_DB_GetGroup(data, GWEN_PATH_FLAGS_CREATE_GROUP, "day");
-
-      dbTransaction=0;
-      DBG_INFO(AQBANKING_LOGDOMAIN, "Starting new day");
-      dbSaldo=GWEN_DB_GetGroup(dbDay, GWEN_PATH_FLAGS_CREATE_GROUP,
-                               "StartSaldo");
-      GWEN_DB_AddGroupChildren(dbSaldo, dbTemplate);
-      if (AHB_SWIFT940_Parse_6_0_2(tg, flags, dbSaldo, cfg)) {
-        DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
-        GWEN_DB_Group_free(dbTemplate);
-	GWEN_Gui_ProgressEnd(progressId);
-        return -1;
-      }
-      else {
-	dbDate=GWEN_DB_GetGroup(dbSaldo, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-				"date");
-      }
-
-      curr=GWEN_DB_GetCharValue(dbSaldo, "value/currency", 0, 0);
-      if (curr) {
-	AHB_SWIFT__SetCharValue(dbTemplate, flags,
-				"value/currency", curr);
-      }
-    }
-    else if (strcasecmp(id, "62F")==0) { /* EndSaldo */
-      GWEN_DB_NODE *dbSaldo;
-
-      /* end current day */
-      dbTransaction=0;
-      if (!dbDay) {
-        DBG_WARN(AQBANKING_LOGDOMAIN, "Your bank does not send an opening saldo");
-        dbDay=GWEN_DB_GetGroup(data, GWEN_PATH_FLAGS_CREATE_GROUP, "day");
-      }
-      dbSaldo=GWEN_DB_GetGroup(dbDay, GWEN_PATH_FLAGS_CREATE_GROUP,
-                               "EndSaldo");
-      GWEN_DB_AddGroupChildren(dbSaldo, dbTemplate);
-      if (AHB_SWIFT940_Parse_6_0_2(tg, flags, dbSaldo, cfg)) {
-        DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
-        GWEN_DB_Group_free(dbTemplate);
-	GWEN_Gui_ProgressEnd(progressId);
-        return -1;
-      }
-      dbDay=0;
-    }
-    else if (strcasecmp(id, "61")==0) {
-      if (!dbDay) {
-	DBG_WARN(AQBANKING_LOGDOMAIN,
-		 "Your bank does not send an opening saldo");
-        dbDay=GWEN_DB_GetGroup(data, GWEN_PATH_FLAGS_CREATE_GROUP, "day");
-      }
-
-      DBG_INFO(AQBANKING_LOGDOMAIN, "Creating new transaction");
-      dbTransaction=GWEN_DB_GetGroup(dbDay, GWEN_PATH_FLAGS_CREATE_GROUP,
-                                     "transaction");
-      GWEN_DB_AddGroupChildren(dbTransaction, dbTemplate);
-      if (dbDate) {
-	GWEN_DB_NODE *dbT;
-
-	/* dbDate is set upon parsing of tag 60F, use it as a default
-	 * if possible */
-	dbT=GWEN_DB_GetGroup(dbTransaction, GWEN_DB_FLAGS_OVERWRITE_GROUPS,
-			     "date");
-        assert(dbT);
-	GWEN_DB_AddGroupChildren(dbT, dbDate);
-      }
-      if (AHB_SWIFT940_Parse_61(tg, flags, dbTransaction, cfg)) {
-        DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
-        GWEN_DB_Group_free(dbTemplate);
-	GWEN_Gui_ProgressEnd(progressId);
-        return -1;
-      }
-    }
-    else if (strcasecmp(id, "86")==0) {
-      if (!dbTransaction) {
-	DBG_WARN(AQBANKING_LOGDOMAIN,
-		 "Bad sequence of tags (86 before 61), ignoring");
-      }
-      else {
-        if (AHB_SWIFT940_Parse_86(tg, flags, dbTransaction, cfg)) {
-          DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
-          GWEN_DB_Group_free(dbTemplate);
-	  GWEN_Gui_ProgressEnd(progressId);
-          return -1;
-        }
-      }
-    }
-    else if (strcasecmp(id, "NS")==0) {
-      if (!dbTransaction) {
-	DBG_DEBUG(AQBANKING_LOGDOMAIN,
-		  "Ignoring NS tags outside transactions");
-      }
-      else {
-	if (AHB_SWIFT940_Parse_NS(tg, flags, dbTransaction, cfg)) {
-	  DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
-	  GWEN_DB_Group_free(dbTemplate);
-	  GWEN_Gui_ProgressEnd(progressId);
-	  return -1;
+    else {
+      if (!ignoreCurrentReport) {
+	if (strcasecmp(id, "25")==0) { /* LocalAccount */
+	  if (AHB_SWIFT940_Parse_25(tg, flags, dbTemplate, cfg)) {
+	    DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
+	    GWEN_DB_Group_free(dbTemplate);
+	    GWEN_Gui_ProgressEnd(progressId);
+	    return -1;
+	  }
+	}
+	else if (strcasecmp(id, "60F")==0) { /* StartSaldo */
+	  GWEN_DB_NODE *dbSaldo;
+	  const char *curr;
+    
+	  /* start a new day */
+	  dbDay=GWEN_DB_GetGroup(data, GWEN_PATH_FLAGS_CREATE_GROUP, "day");
+    
+	  dbTransaction=0;
+	  DBG_INFO(AQBANKING_LOGDOMAIN, "Starting new day");
+	  dbSaldo=GWEN_DB_GetGroup(dbDay, GWEN_PATH_FLAGS_CREATE_GROUP,
+				   "StartSaldo");
+	  GWEN_DB_AddGroupChildren(dbSaldo, dbTemplate);
+	  if (AHB_SWIFT940_Parse_6_0_2(tg, flags, dbSaldo, cfg)) {
+	    DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
+	    GWEN_DB_Group_free(dbTemplate);
+	    GWEN_Gui_ProgressEnd(progressId);
+	    return -1;
+	  }
+	  else {
+	    dbDate=GWEN_DB_GetGroup(dbSaldo, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+				    "date");
+	  }
+    
+	  curr=GWEN_DB_GetCharValue(dbSaldo, "value/currency", 0, 0);
+	  if (curr) {
+	    AHB_SWIFT__SetCharValue(dbTemplate, flags,
+				    "value/currency", curr);
+	  }
+	}
+	else if (strcasecmp(id, "62F")==0) { /* EndSaldo */
+	  GWEN_DB_NODE *dbSaldo;
+    
+	  /* end current day */
+	  dbTransaction=0;
+	  if (!dbDay) {
+	    DBG_WARN(AQBANKING_LOGDOMAIN, "Your bank does not send an opening saldo");
+	    dbDay=GWEN_DB_GetGroup(data, GWEN_PATH_FLAGS_CREATE_GROUP, "day");
+	  }
+	  dbSaldo=GWEN_DB_GetGroup(dbDay, GWEN_PATH_FLAGS_CREATE_GROUP,
+				   "EndSaldo");
+	  GWEN_DB_AddGroupChildren(dbSaldo, dbTemplate);
+	  if (AHB_SWIFT940_Parse_6_0_2(tg, flags, dbSaldo, cfg)) {
+	    DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
+	    GWEN_DB_Group_free(dbTemplate);
+	    GWEN_Gui_ProgressEnd(progressId);
+	    return -1;
+	  }
+	  dbDay=0;
+	}
+	else if (strcasecmp(id, "61")==0) {
+	  if (!dbDay) {
+	    DBG_WARN(AQBANKING_LOGDOMAIN,
+		     "Your bank does not send an opening saldo");
+	    dbDay=GWEN_DB_GetGroup(data, GWEN_PATH_FLAGS_CREATE_GROUP, "day");
+	  }
+    
+	  DBG_INFO(AQBANKING_LOGDOMAIN, "Creating new transaction");
+	  dbTransaction=GWEN_DB_GetGroup(dbDay, GWEN_PATH_FLAGS_CREATE_GROUP,
+					 "transaction");
+	  GWEN_DB_AddGroupChildren(dbTransaction, dbTemplate);
+	  if (dbDate) {
+	    GWEN_DB_NODE *dbT;
+    
+	    /* dbDate is set upon parsing of tag 60F, use it as a default
+	     * if possible */
+	    dbT=GWEN_DB_GetGroup(dbTransaction, GWEN_DB_FLAGS_OVERWRITE_GROUPS,
+				 "date");
+	    assert(dbT);
+	    GWEN_DB_AddGroupChildren(dbT, dbDate);
+	  }
+	  if (AHB_SWIFT940_Parse_61(tg, flags, dbTransaction, cfg)) {
+	    DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
+	    GWEN_DB_Group_free(dbTemplate);
+	    GWEN_Gui_ProgressEnd(progressId);
+	    return -1;
+	  }
+	}
+	else if (strcasecmp(id, "86")==0) {
+	  if (!dbTransaction) {
+	    DBG_WARN(AQBANKING_LOGDOMAIN,
+		     "Bad sequence of tags (86 before 61), ignoring");
+	  }
+	  else {
+	    if (AHB_SWIFT940_Parse_86(tg, flags, dbTransaction, cfg)) {
+	      DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
+	      GWEN_DB_Group_free(dbTemplate);
+	      GWEN_Gui_ProgressEnd(progressId);
+	      return -1;
+	    }
+	  }
+	}
+	else if (strcasecmp(id, "NS")==0) {
+	  if (!dbTransaction) {
+	    DBG_DEBUG(AQBANKING_LOGDOMAIN,
+		      "Ignoring NS tags outside transactions");
+	  }
+	  else {
+	    if (AHB_SWIFT940_Parse_NS(tg, flags, dbTransaction, cfg)) {
+	      DBG_INFO(AQBANKING_LOGDOMAIN, "Error in tag");
+	      GWEN_DB_Group_free(dbTemplate);
+	      GWEN_Gui_ProgressEnd(progressId);
+	      return -1;
+	    }
+	  }
 	}
       }
     }
